/*
 * Copyright (C) 2004, 2005 Oliver Plohmann All rights reserved. Redistribution
 * and use in source and binary forms, with or without modification, are
 * permitted provided that redistributions of source code or in binary form must
 * reproduce the above copyright notice, and the disclaimer that follows. In
 * addition, you are required that you include in the end-user documentation
 * provided with the redistribution and/or in the software itself an
 * acknowledgement equivalent to the following: "This product is based on the
 * Janet software developed by the JANET Project
 * (http://www.almendra.org/janet/)." THIS SOFTWARE IS PROVIDED ``AS IS''
 * AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.IN NO EVENT SHALL THE JANET AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.almendra.commons.events;

import java.util.LinkedList;
import java.util.List;
import java.util.NoSuchElementException;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

import org.almendra.commons.util.StringUtility;

/**
 * @author Oliver
 */
public abstract class AbstractSimpleEventRegistry<HandlerType extends IEventHandler> implements ISimpleEventRegistry<HandlerType>
{
    protected Lock accessProtection = new ReentrantLock();

    /**
     *  
     */
    public AbstractSimpleEventRegistry()
    {
        super();
    }

    public void occured(Event event)
    {
        accessProtection.lock();
        try
        {            
            List<HandlerType> handlers = getHandlers(event.getName());
            if (handlers == null)
                return;
            for (HandlerType handler : handlers)
                handler.occured(event);
        }
        catch (Exception e)
        {
            System.out.println(e.getMessage());
            e.printStackTrace();
            // TODO - write ex to logger
        }
        finally
        {
            accessProtection.unlock();
        }
    }

    public void register(String eventName, HandlerType handler) throws IllegalArgumentException
    {
        checkEventNameValid(eventName);
        accessProtection.lock();
        try
        {            
            List<HandlerType> handlers = getHandlers(eventName);
            if (handlers == null)
            {
                handlers = new LinkedList<HandlerType>();
                putRegistry(eventName, handlers);
            }
            handlers.add(handler);
        }
        finally
        {
            accessProtection.unlock();
        }
    }

    public void deregister(String eventName, HandlerType handler)
    {

        checkEventNameValid(eventName);
        accessProtection.lock();
        try
        {            
            List<HandlerType> handlers = getHandlers(eventName);
            if (handlers == null)
                throwNoSuchEvent(eventName);
            boolean found = handlers.remove(handler);
            if (!found)
                throwNoSuchEvent(eventName);
            if (handlers.isEmpty())
                registryRemove(eventName);
        }
        finally
        {
            accessProtection.unlock();
        }
    }

    public abstract void putRegistry(String eventName, List<HandlerType> handlers);
    
    /**
     * @param eventName
     */
    protected void throwNoSuchEvent(String eventName)
    {
        throw new NoSuchElementException("no handler registered for event " + eventName + "!"); //$NON-NLS-1$ //$NON-NLS-2$
    }

    protected void checkEventNameValid(String eventName)
    {
        if (!StringUtility.isValid(eventName))
            throw new IllegalArgumentException("no name specified for event"); //$NON-NLS-1$
    }

    protected abstract List<HandlerType> getHandlers(String eventName);

    protected abstract void registryRemove(String eventName);
}