/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source code or
 * in binary form must reproduce the above copyright notice, and the disclaimer
 * that follows.
 * 
 * In addition, you are required that you include in the end-user documentation
 * provided with the redistribution and/or in the software itself an
 * acknowledgement equivalent to the following: "This product is based on the
 * Janet software developed by the JANET Project
 * (http://www.almendra.org/janet)."
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE JDOM
 * AUTHORS OR THE PROJECT CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */

package org.almendra.commons.events;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

/**
 * @author oplohmann
 *
 */
public class EventRegistryCore<T extends EventNotifier>
{
	/**
	 * Registry of events with event name and event notifiers for the event as
	 * values.
	 */
	protected Map<String, List<T>> registry = new HashMap<String, List<T>>();
	protected Lock accessProtection = new ReentrantLock(true);
	
    /* (non-Javadoc)
     * @see org.almendra.commons.janet.cas.events.ILocalClusterEventRegistryRemote#getHandlerCount(java.lang.String)
     */
    public int getHandlerCount(String eventName)
    {
        accessProtection.lock();
        try
        {
            int count = 0;
        	List<T> handlers = registry.get(eventName);
        	if(handlers == null)
        		return count;
        	for(EventNotifier notifier : handlers)
        		count += notifier.getEventHandlers().get().size();
        	return count;
        }
    	finally
    	{
    	    accessProtection.unlock();
    	}
    }

    /**
     * Notify the event registry that an event has occurred. The event registry
     * will notify all event handlers registered for the event.
     * 
     * @param event
     *            event that has occurred specified by its name.
     */
    public void occured(Event event)
    {
    	accessProtection.lock();
    	try
    	{
    	    List<T> handlers = registry.get(event.getName());
        	if (handlers == null)		
        		return;
        	for(EventNotifier notifier : handlers)
        		notifier.occured(event);
    	}
    	finally
    	{
    	    accessProtection.unlock();
    	}
    }

    /**
     * Answer whether an event is registred with the event registry.
     * 
     * @param eventName
     *            name of the event
     * @return
     */
    public boolean isRegistered(String eventName)
    {
    	accessProtection.lock();
    	try
    	{
    	    return registry.containsKey(eventName);
    	}
    	finally
    	{
    	    accessProtection.unlock();
    	}
    }
}
