/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.almendra.org/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE JANET AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
 
package org.almendra.commons.util.collection;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

/**
 * <code>CollectionUtility</code> provides several utility methods to deal with collentions
 * not found in <code>Collections</code> or other classes of the Java collection framework.
 * 
 * @author Oliver Plohmann
 */

public class CollectionUtility
{
	/**
	 * Creates a new <code>CollectionUtility</code> object. Not used since all methods are static.
	 */
	private CollectionUtility()
	{
		super();
	}
	
	/**
	 * Answer all elements that appear in more than once the 
	 * collections contained in <code>listOfLists</code>.
	 * @param listOfLists list that contains lists
	 * @return set of duplicate elements
	 */
	public static <T> Set<T> duplicates(List<List<T>> listOfLists)
	{
		Set<T> duplicates = new HashSet<T>();
		Set<T> allObjects = new HashSet<T>();
		
		for(List<T> list : listOfLists)
		{
			for(T t : list)
			{
				if(allObjects.contains(t))
					duplicates.add(t);
				else
					allObjects.add(t);
			}
		}
		return duplicates;
	}

	/**
	 * Convert a <code>Collection</code> to a <code>Set</code>.
	 * @param collection to be converted
	 * @return 
	 */
	public static <T> Set<T> toSet(Collection<T> collection)
	{
		Set<T> set = new HashSet<T>();
		for(T t : collection)
			set.add(t);
		return set;
	}
	
	/**
	 * Convert a <code>Collection</code> to a <code>List</code>.
	 * @param collection to be converted
	 * @return
	 */
	public static <T> List<T> toList(Collection<T> collection)
	{
		List<T> list = new ArrayList<T>();
		for(T t : collection)
			list.add(t);
		return list;
	}

	/**
	 * Answer a <code>List</code> containing an object.
	 * @param object to be contained in a list
	 * @return list with contained object
	 */
	public static <T> List<T> toList(T object)
	{
		List<T> list = new ArrayList<T>();
		list.add(object);
		return list;
	}
	
	/**
	 * Answer an array of strings containing all the strings of the collection argument.
	 * @param collection the argument collection containing strings
	 * @return an array of strings
	 */
	public static String[] toStringArray(Collection<String> collection)
	{
		int size = collection.size();
		String[] array = new String[size];
		Iterator<String> it = collection.iterator();
		for (int i = 0; i < size; i++)
			array[i] = it.next();
		return array;
	}
	
}
