/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.almendra.org/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE JANET AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.commons.util.globalId;

import java.math.BigInteger;
import java.util.Random;

import org.almendra.commons.util.inet.InetUtility;


/**
 * Generates globally unique ids. These ids contain the name of the local machine which is
 * assumed to be unique within a network, a running counter and a timestamp. The timestamp is 
 * used to be sure that the id is unique even after the running counter has had overflow and
 * therefore has been reset to 0.
 * 
 * The counter has meanwhile been changed from an int to a BigInteger. Overflow is now very 
 * unlikely to occur. The timestamp is kept as there can be several JVMs on the same 
 * machine using the same code to generate unique ids.
 * 
 * @author Oliver Plohmann
 */

public class UniqueIdGenerator
{
	/**
	 * The running counter which is part of an id.
	 */
	protected static BigInteger Count = BigInteger.ZERO; //$NON-NLS-1$
	
	/**
	 * The name of the local machine assumed unique within a network.
	 */
	protected static String HostName = null;

	protected static Random random = new Random(System.currentTimeMillis());
	
	/**
	 * Creates a new <code>UniqueIdGenerator</code> object.
	 */
	public UniqueIdGenerator()
	{
		super();
	}

	/**
	 * Retrieve the name of the local machine which is part of an id.
	 * @return
	 */
	protected static String getHostName()
	{
		if (HostName == null)
		{
			HostName = InetUtility.getHostName();
		}
		return HostName;
	}

	/**
	 * Return a new unique id.
	 * @return
	 */
	public static synchronized UniqueId next()
	{
		return next(getHostName());
	}

	/**
	 * Return a new unique id. The user can specify a different prefix than the local machine
	 * name. The user is then responsible for the id to be unique. 
	 * @param prefix the id prefix used instead of the local machine name 
	 * @return the unique id
	 */
	public static synchronized UniqueId next(String prefix)
	{
		String id = String.valueOf(prefix + "-" + Count + "-" + getMillisWithRandom()); //$NON-NLS-1$ //$NON-NLS-2$
		Count = Count.add(BigInteger.ONE);
		return new UniqueId(id);
	}
	
	/**
	 * Return a new counter that has not been used before.
	 * @return
	 */
	public static String getNewCount()
	{
		String count = Count.toString();
		Count = Count.add(BigInteger.ONE);
		return count;
	}
	
	/**
	 * Return a new counter that has not been used before.
	 * @return
	 */
	public static String getMillisWithRandom()
	{
		return System.currentTimeMillis() + "-" + random.nextInt(5000); //$NON-NLS-1$
	}

}
