/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source code or
 * in binary form must reproduce the above copyright notice, and the disclaimer
 * that follows.
 * 
 * In addition, you are required that you include in the end-user documentation
 * provided with the redistribution and/or in the software itself an
 * acknowledgement equivalent to the following: "This product is based on the
 * Janet software developed by the JANET Project
 * (http://www.almendra.org/janet/)."
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE JDOM
 * AUTHORS OR THE PROJECT CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */

package org.almendra.commons.util.logging;

import java.io.Serializable;

import org.almendra.commons.util.AssertUtility;
import org.almendra.commons.util.StringUtility;
import org.almendra.commons.util.TimeUtility;
import org.almendra.commons.util.collection.ArrayUtility;
import org.almendra.commons.util.xml.XMLUtility;


// TODO - code not yet documented

/**
 * @author Oliver Plohmann
 */
public class LogEntry implements Serializable
{
	private static final long serialVersionUID = -4366265490464794396L;
	
	public static final String LOG_ENTRY_HEADER = "<LogEntry"; //$NON-NLS-1$
	public static final String LOG_ENTRY_TAIL = "/>"; //$NON-NLS-1$
	
	public static final String TAG_EVENT = "event"; //$NON-NLS-1$
	public static final String TAG_ARG = "arg"; //$NON-NLS-1$
	public static final String TAG_TIME = "time"; //$NON-NLS-1$
	public static final String TAG_TYPE = "type"; //$NON-NLS-1$
	
	protected int level = -1;
	protected String argument = null;
	protected String time = null;
	protected String event = null;
	protected String logMessage = null;

	protected LogEntry()
	{
		super();
	}

	public LogEntry(int level)
	{
		super();
		this.level = level;
	}

	public LogEntry(int level, String argument)
	{
		super();
		this.level = level;
		this.argument = argument;
	}

	/**
	 * Creates a new <code>LogEntry</code> object.
	 * @param entry
	 */
	public LogEntry(LogEntry entry)
	{
		super();
		fillFromEntryTemplate(entry);
	}
	
	public String toLogString()
	{
		String separator = getLogStringSeparator();
		StringBuffer buffer = new StringBuffer();

		if (event != null)
		{
			buffer.append(TAG_EVENT + "=" + StringUtility.print(event)); //$NON-NLS-1$
			buffer.append(separator);
		}
		if (argument != null)
		{
			buffer.append(TAG_ARG + "=" + StringUtility.print(argument)); //$NON-NLS-1$
			buffer.append(separator);
		}
		if (time != null)
		{
			buffer.append(TAG_TIME + "=" + StringUtility.print(time)); //$NON-NLS-1$
			buffer.append(separator);
		}

		String logString = buffer.toString();

		if (logString.length() == 0)
			return logString;

		if (logString.endsWith(separator))
			logString = logString.substring(0, logString.length() - separator.length());

		return logString;
	}

	protected String getLogStringSeparator()
	{
		return " "; // previously " - " //$NON-NLS-1$
	}

	/**
	 * @return
	 */
	public int getLevel()
	{
		return level;
	}

	/**
	 * @param i
	 */
	public void setLevel(int i)
	{
		level = i;
	}

	/**
	 * @return
	 */
	public String getTime()
	{
		return time;
	}

	/**
	 * @param string
	 */
	public void setType(String type, String[] levelLabels)
	{
		setLevel(ArrayUtility.indexOf(levelLabels, type));
	}

	/**
	 * @return
	 */
	public String getEvent()
	{
		return event;
	}

	/**
	 * @param string
	 */
	public void setEvent(String string)
	{
		event = string;
	}

	/**
	 * @return
	 */
	public String getArgument()
	{
		return argument;
	}

	/**
	 * @param string
	 */
	public void setArgument(String string)
	{
		argument = string;
	}

	/**
	 * Fill a <code>LogEntry</code> with the values from another <code>LogEntry</code> object.
	 * 
	 * @param entry
	 */
	protected void fillFromEntryTemplate(LogEntry entry)
	{
		setTime(TimeUtility.now());
		setArgument(entry.getArgument());
		setEvent(entry.getEvent());
		setLevel(entry.getLevel());
	}

	/**
	 * @return
	 */
	public String getLogMessage()
	{
		return logMessage;
	}

	/**
	 * @param levelLabels
	 */
	public void setLogMessage(String[] levelLabels)
	{
		logMessage = LOG_ENTRY_HEADER + " " + TAG_TYPE + "=" + StringUtility.print(levelLabels[getLevel()]) + " " + toLogString() + LOG_ENTRY_TAIL; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$			
	}

	/**
	 * @param entryString
	 */
	public void fillFromEntryTemplate(String entryString, String[] levelLabels)
	{
		// TODO -- not yet implemented
		setType(XMLUtility.getXMLValue(entryString, TAG_TYPE), levelLabels);
		setTime(XMLUtility.getXMLValue(entryString, TAG_TIME));
		setEvent(XMLUtility.getXMLValue(entryString, TAG_EVENT));
		setArgument(XMLUtility.getXMLValue(entryString, TAG_ARG));
	}
	/**
	 * @param time The time to set.
	 */
	public void setTime(String time)
	{
		this.time = time;
	}

	/**
	 * @return
	 */
	protected boolean mustBeSubscribed()
	{
		AssertUtility.assertTrue(level != -1);
		if(getLevel() == Logger.FREE_TEXT)
			return false;
		if(getLevel() == Logger.FATAL)
			return false;
		if(getLevel() == Logger.ERROR)
			return false;
		if(getLevel() == Logger.EXCEPTION)
			return false;
		return true;
	}
	
	public static LogEntry getNewObject()
	{
	    return new LogEntry();
	}

}
