/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.almendra.org/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.IN NO EVENT SHALL THE JANET AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
package org.almendra.commons.util.swing;

import java.util.LinkedList;
import java.util.List;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

import javax.swing.JTextArea;
import javax.swing.text.BadLocationException;

import org.almendra.commons.util.StringUtility;
import org.almendra.commons.util.collection.Association;
import org.almendra.commons.util.collection.StringList;
import org.almendra.commons.util.logging.ErrorLogger;


public class TextAreaPanelController
{
	public static final String LABEL = "TextAreaPanelController"; //$NON-NLS-1$
	
	protected JTextArea textArea = null;
	protected int clearTextTabAfterLines = -1;
	protected int linesCount = -1;
	protected Lock accessProtection = new ReentrantLock(true);
	protected List<Association<Integer, Integer>> headerOffsets = new LinkedList<Association<Integer, Integer>>();
	protected int spaces = -1;
	protected List<String> headerTitles = new StringList();
	
	/**
	 * Creates a new <code>TextAreaPanelController</code> object.
	 */
	protected TextAreaPanelController()
	{
		super();
	}
	
	/**
	 * Creates a new <code>TextAreaPanelController</code> object.
	 */
	public TextAreaPanelController(JTextArea textArea)
	{
		super();
		this.textArea = textArea;
	}
	
	/**
	 * Creates a new <code>TextAreaPanelController</code> object.
	 */
	public TextAreaPanelController(JTextArea textArea, int clearTextTabAfterLines)
	{
		super();
		this.textArea = textArea;
		this.clearTextTabAfterLines = clearTextTabAfterLines;
	}
	
	public void appendBlankLine()
	{
		append("\n"); //$NON-NLS-1$
	}
	
	public void append(String text)
	{
	    accessProtection.lock();
		try
		{			
			countLines(text);
			textArea.append(text);
		}
		finally
		{
			accessProtection.unlock();
		}			
	}
	
	public void insertBelowHeader(List<String> values)
	{
		if(headerOffsets.isEmpty())
			return;
		
		accessProtection.lock();
		try
		{			
			insert(getLine(values), 2);			
		}
		finally
		{
			accessProtection.unlock();
		}
	}
	
	/**
	 * @param values
	 * @return
	 */
	protected String getLine(List<String> values)
	{		
		StringBuffer buffer = new StringBuffer();
		int size = values.size();
		String tab = StringUtility.spaces(spaces);
		for (int i = 0; i < size; i++)
		{
			String value = values.get(i);
			if(headerOffsets.size() >= size)
			{
				Association<Integer, Integer> offsets = headerOffsets.get(i);				
				int start = offsets.getKey().intValue();
				int end = offsets.getValue().intValue();
				int fillSpace = end - start - value.length();
				String fillSpaces = StringUtility.spaces(fillSpace / 2);
				buffer.append(tab);
				buffer.append(fillSpaces);
				buffer.append(value);
				buffer.append(fillSpaces);
			}
		}		
		return buffer.toString();
	}

	public void insert(String text, int line)
	{
	    accessProtection.lock();
		try
		{
			countLines(text);
			if(text.length() == 0)
				return;
			int lineCount = textArea.getLineCount();
			for (int i = lineCount; i <= line; i++)
				textArea.append("\n"); //$NON-NLS-1$
			int pos = textArea.getLineEndOffset(line);
			textArea.insert(text + "\n", pos); //$NON-NLS-1$
		}
		catch (BadLocationException e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
			ErrorLogger.exception(e);
		}
		finally
		{
			accessProtection.unlock();
		}				
	}
	
	public void prepend(String text)
	{
		accessProtection.lock();
		try
		{
			countLines(text);
			int firstLine = 1;
			if(textArea.getLineCount() < firstLine)
				textArea.append("\n"); //$NON-NLS-1$
			textArea.insert(text, 0);
		}
		finally
		{
			accessProtection.unlock();
		}				
	}

	public void clear()
	{
		clear(true);
	}
	
	public void clear(boolean resetHeader)
	{
		accessProtection.lock();
		try
		{
			linesCount = 0;
			textArea.setText(""); //$NON-NLS-1$
			if(resetHeader)
				resetHeader();				
		}
		finally
		{
			accessProtection.unlock();
		}		
	}
	
	/**
	 * 
	 */
	protected void resetHeader()
	{
		if(!hasHeader())
			return;
		setHeader(headerTitles, spaces);
	}

	/**
	 * @param text
	 */
	protected void countLines(String text)
	{
		if(clearTextTabAfterLines != -1)
		{
			int newLinesCount = StringUtility.linesCount(text); //$NON-NLS-1$
			if(linesCount + newLinesCount > clearTextTabAfterLines)
			{
				linesCount = 0;
				clear();
			}
			linesCount += newLinesCount;
		}
	}

	public boolean hasHeader()
	{
		return headerTitles.size() > 0;
	}
	
	/**
	 * @param selectedNodeNames
	 */
	public void setHeader(List<String> headerTitles, int spaces)
	{
	    accessProtection.lock();
		try
		{			
			this.spaces = spaces;
			this.headerTitles = headerTitles;
			headerOffsets = new LinkedList<Association<Integer, Integer>>();
			
			if(headerTitles.isEmpty())
			{
				clear(false);
				return;
			}
			
			String tab = StringUtility.spaces(spaces);			
			clear(false);
			appendBlankLine();			
			StringBuffer buffer = new StringBuffer();
			for(String headerTitle : headerTitles)
			{
				buffer.append(tab);
				int start = buffer.length();
				buffer.append(headerTitle);
				int end = buffer.length();
				headerOffsets.add(new Association<Integer, Integer>(new Integer(start), new Integer(end)));
			}
			insert(buffer.toString(), 1);
			appendBlankLine();
		}
		finally
		{
			accessProtection.unlock();
		}			
	}
}
