/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.almendra.org/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE JANET AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.janet.ade.observer.simulation.interpreters;

import java.io.File;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Map;

import org.almendra.commons.util.AssertUtility;
import org.almendra.commons.util.StringUtility;
import org.almendra.commons.util.TimeUtility;
import org.almendra.commons.util.logging.ErrorLogger;
import org.almendra.janet.ade.executor.ExecutorAnchor;
import org.almendra.janet.ade.observer.ObserverAnchor;
import org.almendra.janet.ade.observer.simulation.ObserverAnchorSimulation;
import org.almendra.janet.cas.scheduling.CommandAccessor;
import org.almendra.janet.cas.scheduling.IInterpreter;


public class ObserveWorkstationLoadInterpreter
		extends
			org.almendra.janet.ade.observer.interpreters.ObserveWorkstationLoadInterpreter implements IInterpreter
{
	/**
	 * Creates a new <code>ObserveWorkstationLoadInterpreter</code> object.
	 */
	public ObserveWorkstationLoadInterpreter()
	{
		super();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.almendra.commons.janet.ade.observer.interpreters.ObserveWorkstationLoadInterpreter#getCurrentCPULoadByProcesses()
	 */
	protected Map<String, Integer> getCurrentCPULoadByProcesses(CommandAccessor cmdAccessor)
	{
		Map<String, Integer> queueSizeCategories = getQueueSizeCategories(cmdAccessor);
		return translateIntoPercentages(queueSizeCategories);
		
	}

	/**
	 * @param queueSizeCategories
	 * @return
	 */
	protected Map<String, Integer> translateIntoPercentages(Map<String, Integer> queueSizeCategories)
	{
		if(queueSizeCategories.isEmpty())
			return new HashMap<String, Integer>();
		if(!queueSizeCategories.containsKey(ObserverAnchor.ALL_OTHERS_LABEL))
			return new HashMap<String, Integer>();
		int allOthers = queueSizeCategories.get(ObserverAnchor.ALL_OTHERS_LABEL).intValue();
		int remainCPUPower = 100 - allOthers;
		AssertUtility.assertTrue(remainCPUPower >= 0);
		Map<String, Integer> remainingQueueSizeCategories = new HashMap<String, Integer>(queueSizeCategories);
		remainingQueueSizeCategories.remove(ObserverAnchor.ALL_OTHERS_LABEL);
		int loadedNodesCount = 0;
		for(Integer value : remainingQueueSizeCategories.values())
		{
			if(value.intValue() != 0)
				loadedNodesCount++;
		}
		Map<String, Integer> percentages = new HashMap<String, Integer>();
		int share = 0;
		if(loadedNodesCount != 0)
			share = remainCPUPower / loadedNodesCount;
		
		for(String key : queueSizeCategories.keySet())
		{
			if(key.equals(ObserverAnchor.ALL_OTHERS_LABEL))
				percentages.put(ObserverAnchor.ALL_OTHERS_LABEL, new Integer(allOthers));
			else
				percentages.put(key, new Integer(share));
		}
		return percentages;
	}

	/**
	 * @param cmdAccessor
	 * @return
	 */
	protected Map<String, Integer> getQueueSizeCategories(CommandAccessor cmdAccessor)
	{
		Map<String, Integer> queueSizeCatgoriesByNodeName = new HashMap<String, Integer>();
		ObserverAnchorSimulation anchor = (ObserverAnchorSimulation) getAnchor(cmdAccessor);
		queueSizeCatgoriesByNodeName.put(ObserverAnchor.ALL_OTHERS_LABEL, new Integer(anchor.getSimulatedNonJanetCPULoad()));

		for(String nodeName : anchor.getNodeNames())
		{
			int queueSizeCategory = getQueueSizeCategory(nodeName);
			if(queueSizeCategory != -1)
				queueSizeCatgoriesByNodeName.put(nodeName, new Integer(queueSizeCategory));
		}
		return queueSizeCatgoriesByNodeName;
	}

	protected int getQueueSizeCategory(String nodeName)
	{
		try
		{
			File file = new File(nodeName + ExecutorAnchor.INFO_FILE_SUFFIX);
			if (!file.exists())
				return -1;
			java.io.FileInputStream fis = new java.io.FileInputStream(file);
			java.io.ObjectInputStream ois = new java.io.ObjectInputStream(fis);
			String queueSizeCategoryString = ois.readUTF();
			ois.close();
			fis.close();
			Integer queueSizeCategorInt = StringUtility.toInteger(queueSizeCategoryString);
			return queueSizeCategorInt.intValue();
		}
		catch (Exception e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
			ErrorLogger.exception(e);
		}		
		return -1;
	}

	protected boolean infoOutOfDate(String date)
	{
		GregorianCalendar cal = TimeUtility.fromShort(date);
		long now = System.currentTimeMillis();
		long calInMillis = cal.getTimeInMillis();
		if(now - calInMillis < 0)
			return true;
		if(now - calInMillis > 10000)	// 10 secs
			return true;
		return false;
	}
	
	/**
	 * @return Returns the anchor.
	 */
	public ObserverAnchor getAnchor(CommandAccessor cmdAccessor)
	{
		return (ObserverAnchor) cmdAccessor.getApplicationObjectSpace().get(ObserverAnchor.LABEL);
	}
}
