/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source code or
 * in binary form must reproduce the above copyright notice, and the disclaimer
 * that follows.
 * 
 * In addition, you are required that you include in the end-user documentation
 * provided with the redistribution and/or in the software itself an
 * acknowledgement equivalent to the following: "This product is based on the
 * Janet software developed by the JANET Project
 * (http://www.almendra.org/janet/)."
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE JDOM
 * AUTHORS OR THE PROJECT CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */

package org.almendra.janet.cas.demo.fibonacci;

import java.math.BigInteger;
import java.rmi.AccessException;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.rmi.registry.Registry;

import org.almendra.commons.util.inet.RMIUtility;
import org.almendra.commons.util.logging.LoggerAccessor;
import org.almendra.commons.util.swing.*;
import org.almendra.janet.cas.node.AgentPath;
import org.almendra.janet.cas.node.Application;
import org.almendra.janet.cas.node.Capability;
import org.almendra.janet.cas.node.LogEntry;
import org.almendra.janet.cas.scheduling.CommandHistorySummary;


/**
 * @author Oliver
 *  
 */
public class FibonacciUtility
{
	public static final String NODE_NAME = "CAS.FIBONACCI Node"; //$NON-NLS-1$
	public static final String APP_NAME = "CAS_FIBONACCI"; //$NON-NLS-1$
	public static final String CAP_NAME = "CORE"; //$NON-NLS-1$
	public static final String AGENT_NAME = "CALCULATOR"; //$NON-NLS-1$

	/**
	 * Creates a new <code>FibonacciUtility</code> object.
	 */
	public FibonacciUtility()
	{
		super();
	}

	/**
	 * @param args
	 */
	protected static void clearRegistry(String[] args)
	{
		boolean clearRegistry = false;
		for (int i = 0; i < args.length; i++)
		{
			if (args[i].equalsIgnoreCase("-clearRegistry")) //$NON-NLS-1$
			{
				clearRegistry = true;
				break;
			}
		}

		if (!clearRegistry)
			return;

		int port = Registry.REGISTRY_PORT;
		String hostname = null;
		
		for (int i = 0; i < args.length; i++)
		{
			if (args[i].indexOf("-hostname:") != -1) //$NON-NLS-1$
			{
				String value = args[i];
				hostname = value.substring(value.indexOf(":") + 1, value.length()); //$NON-NLS-1$
				break;
			}
		}
		
		for (int i = 0; i < args.length; i++)
		{
			if (args[i].indexOf("-port:") != -1) //$NON-NLS-1$
			{
				String value = args[i];
				value = value.substring(value.indexOf(":") + 1, value.length()); //$NON-NLS-1$
				port = Integer.parseInt(value);
				break;
			}
		}
		
		Registry registry = null;

		try
		{
			registry = RMIUtility.getRegistry(hostname, port);
		}
		catch (RemoteException e)
		{
			// no registry found - clearing it becomes unnecessary
			return;
		}

		String[] list = null;
		try
		{
			list = registry.list();
		}
		catch (AccessException e1)
		{
			System.out.println(e1.getMessage());
			e1.printStackTrace();
		}
		catch (RemoteException e1)
		{
			System.out.println(e1.getMessage());
			e1.printStackTrace();
		}

		for (int i = 0; i < list.length; i++)
		{
			try
			{
				registry.unbind(list[i]);
			}
			catch (AccessException ex)
			{
				System.out.println(ex.getMessage());
				ex.printStackTrace();
			}
			catch (RemoteException ex)
			{
				System.out.println(ex.getMessage());
				ex.printStackTrace();
			}
			catch (NotBoundException ex)
			{
				System.out.println(ex.getMessage());
				ex.printStackTrace();
			}
		}

		System.out.println("cleared registry"); //$NON-NLS-1$
	}

	public static Application getApplication()
	{
		Application app = new Application(APP_NAME);
		Capability cap = new Capability(app, CAP_NAME, AGENT_NAME);
		try
		{
			cap.add(new FibonacciInterpreter());
		}
		catch (Exception e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
			return null;
		}
		return app;
	}

	public static AgentPath getAgentPath(String nodeName)
	{
		return new AgentPath(nodeName, FibonacciUtility.APP_NAME, FibonacciUtility.CAP_NAME, FibonacciUtility.AGENT_NAME);
	}
	
	/**
	 * Don't show the entire Fibonacci number. Some eclipse versions older than version 3.0 cause
	 * the entire system to hang if a string, output to the same console line, is too long. 
	 * @param fibonacciSize
	 * @param result
	 */
	public static <T> void displayResult(int fibonacciSize, BigInteger result, LoggerAccessor loggerOrNull, CommandHistorySummary summaryOrNull, JTextPaneHolder textPaneHolder)
	{
		String resultString = result.toString();
		int len = resultString.length();
		StringBuffer buffer = new StringBuffer();
		buffer.append("\nFibonacciCommand(" + fibonacciSize + "): \n"); //$NON-NLS-1$ //$NON-NLS-2$
		buffer.append("- a number with " + len + " digits\n"); //$NON-NLS-1$ //$NON-NLS-2$
		buffer.append("- first 100 digits are: " + resultString.substring(0, Math.min(resultString.length(), 100)) + "\n"); //$NON-NLS-1$ //$NON-NLS-2$
		
		if(summaryOrNull != null)
		{	
			buffer.append("- systemStayTime=" + summaryOrNull.getSystemStayTime()); //$NON-NLS-1$
			buffer.append(" / travelTime=" + summaryOrNull.getTravelTime()); //$NON-NLS-1$
			buffer.append(" / executionTime=" + summaryOrNull.getExecutionTime()); //$NON-NLS-1$
			buffer.append(" / totalTime=" + summaryOrNull.getTotalTime()); //$NON-NLS-1$
			buffer.append("\n"); //$NON-NLS-1$
		}
		
		if(textPaneHolder != null)
			textPaneHolder.addText(buffer.toString());
		
		if(loggerOrNull == null)
			System.out.println(buffer.toString());
		else
		{
			LogEntry entry = (LogEntry) loggerOrNull.getNewLogEntry();
			entry.setArgument(buffer.toString());
			loggerOrNull.freeText(entry);
		}
	}
}
