/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source code or
 * in binary form must reproduce the above copyright notice, and the disclaimer
 * that follows.
 * 
 * In addition, you are required that you include in the end-user documentation
 * provided with the redistribution and/or in the software itself an
 * acknowledgement equivalent to the following: "This product is based on the
 * Janet software developed by the JANET Project
 * (http://www.almendra.org/janet/)."
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE JDOM
 * AUTHORS OR THE PROJECT CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */

package org.almendra.janet.cas.agents;

import java.rmi.RemoteException;
import java.util.List;
import java.util.Set;

import org.almendra.commons.events.IHistorizableCallbackHandler;
import org.almendra.commons.threads.FutureResult;
import org.almendra.commons.util.AssertUtility;
import org.almendra.commons.util.StringUtility;
import org.almendra.commons.util.collection.StringList;
import org.almendra.commons.util.collection.StringSet;
import org.almendra.commons.util.logging.ILoggable;
import org.almendra.janet.cas.node.AbstractApplication;
import org.almendra.janet.cas.node.AgentPath;
import org.almendra.janet.cas.node.IAgentDispatcher;
import org.almendra.janet.cas.node.IExecutorAnchor;
import org.almendra.janet.cas.node.InvalidCommandException;
import org.almendra.janet.cas.node.LocalCapabilityPath;
import org.almendra.janet.cas.node.LogEntry;
import org.almendra.janet.cas.node.Logger;
import org.almendra.janet.cas.scheduling.Acknowledge;
import org.almendra.janet.cas.scheduling.CommandAckEnvelope;
import org.almendra.janet.cas.scheduling.CommandCallbackEnvelope;
import org.almendra.janet.cas.scheduling.CommandEnvelope;
import org.almendra.janet.cas.scheduling.CommandReplyEnvelope;
import org.almendra.janet.cas.scheduling.ICommand;
import org.almendra.janet.cas.scheduling.ICommandEnvelope;
import org.almendra.janet.cas.scheduling.IWorkLoadBalancingCommand;
import org.almendra.janet.cas.scheduling.IWorkLoadCommand;
import org.almendra.janet.cas.scheduling.IWorkLoadSharingCommand;
import org.almendra.janet.cas.spaces.IObjectSpace;


public class AgentProxy extends AbstractAgentProxy implements IAgentProxy, ILoggable
{
	public static final String LOG_EVENT_SENT_COMMAND = "SENT_COMMAND"; //$NON-NLS-1$
	public static final String LOG_EVENT_NO_ALTERNATIVE_AGENT_FOR_WORKLOAD_SHARING = "NO_ALTERNATIVE_AGENT_FOR_WORKLOAD_SHARING"; //$NON-NLS-1$
	public static final String LOG_EVENT_NO_WORKLOAD_BALANCING_POSSIBLE = "NO_WORKLOAD_BALANCING_POSSIBLE"; //$NON-NLS-1$
	public static final String LOG_EVENT_NO_WORLOAD_SHARING_AVAILABLE = "NO_WORLOAD_SHARING_AVAILABLE"; //$NON-NLS-1$
	public static final String LOG_EVENT_NO_OBSERVER_ON_SAME_WORKSTATION = "NO_OBSERVER_ON_SAME_WORKSTATION"; //$NON-NLS-1$

	protected AbstractApplication application = null;
	protected AgentPath agentPath = null;
	protected IAgentDispatcher agentDispatcher = null;

	/**
	 * Creates a new <code>AgentProxy</code> object.
	 *  
	 */
	protected AgentProxy(Logger logger)
	{
		super(logger);
	}

	/**
	 * Creates a new <code>AgentProxy</code> object.
	 *  
	 */
	public AgentProxy(AbstractApplication application, IAgentDispatcher agentDispatcher, AgentPath agentPath, Logger logger)
	{
		super();
		this.application = application;
		this.agentDispatcher = agentDispatcher;
		this.agentPath = agentPath;
		originNodeName = application.getNode().getName();
		setLogger(logger);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.almendra.commons.janet.cas.logger.node.IAgent#accept(org.almendra.commons.janet.cas.logger.node.CommandEnvelope)
	 */
	public void accept(ICommand command) throws RemoteException, InvalidCommandException
	{
		accept(command, -1);
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.almendra.commons.janet.cas.logger.node.IAgent#accept(org.almendra.commons.janet.cas.logger.node.CommandEnvelope)
	 */
	public void accept(ICommand command, int kind) throws RemoteException, InvalidCommandException
	{
		checkValidity(command);
		ICommandEnvelope env = getCommandEnvelope(agentPath, application.getNode().getName(), command);
		env.setKind(kind);
		env.aboutToBeSent();
		logSentCommand(env);
		env.setSendTime();
		agentDispatcher.accept(env);
	}

	/**
	 * @param command
	 */
	protected void logSentCommand(ICommandEnvelope commandEnvelope)
	{
		if (logger.isSubscribed(LOG_EVENT_SENT_COMMAND))
		{
			LogSentCommand(commandEnvelope, logger);
		}
	}
	
	public static void LogSentCommand(ICommandEnvelope commandEnvelope, Logger logger)
	{
		// TODO - to be removed when the workaround in CommandResponseEnvelope.sendBack()has been removed
		LogEntry entry = logger.getNewLogEntry();
		entry.setEvent(LOG_EVENT_SENT_COMMAND);
		String destination = commandEnvelope.getQualifiedDestinationAgentName();
		String origin = commandEnvelope.getOriginDestinationAgentName();
		StringBuffer buffer = new StringBuffer();
		buffer.append("{" + StringUtility.reversePackageName(commandEnvelope.getCommand().getQualifiedName())); //$NON-NLS-1$
		buffer.append("(id:" + commandEnvelope.getGlobalId().getValue() + ")}"); //$NON-NLS-1$ //$NON-NLS-2$
		buffer.append("{FROM:" + origin + "}{TO:" + destination + "}"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		entry.setArgument(buffer.toString());
		logger.info(entry);
	}

	/**
	 * Send <code>commmand</code> to an agent expecting a result to be sent
	 * back.
	 * 
	 * @param command
	 *            to be sent to another agent
	 * @param replyToAgentPath
	 *            path to agent that should receive result
	 * @return a future that blocks until the result has arrived or times out.
	 * @throws IllegalArgumentException
	 */
	public void accept(ICommand command, FutureResult futureResult) 
	throws IllegalArgumentException, RemoteException, InvalidCommandException
	{
		checkValidity(command);
		String futureId = application.getNode().getUniqueId().getValue();
		CommandReplyEnvelope env = getCommandReplyEnvelope(agentPath, application.getNode().getName(), command, futureId);
		application.getObjectSpace().put(futureId, futureResult);
		env.setReplyToAgentPath(agentPath.getReplyPath(originNodeName));
		env.aboutToBeSent();
		logSentCommand(env);
		env.setSendTime();
		agentDispatcher.accept(env);
	}

	/**
	 * @param ack
	 * @param env
	 */
	protected void setReplyAgentPath(Acknowledge ack, CommandAckEnvelope env)
	throws IllegalArgumentException
	{
		AgentPath ackAgentPath = ack.getAckAgent();
		if(ackAgentPath == null)
			ackAgentPath = agentPath.getReplyPath(originNodeName);
		if(!application.hasAgent(ackAgentPath))
			throw new IllegalArgumentException("application " + application.getName() + " has no such agent " + ackAgentPath); //$NON-NLS-1$ //$NON-NLS-2$
		env.setReplyToAgentPath(ackAgentPath);
	}

	/**
	 * @param sourceDestinationAgentPath
	 * @param originNodeName
	 * @param command
	 * @return
	 */
	protected CommandAckEnvelope getCommandAckEnvelope(
		AgentPath sourceDestinationAgentPath,
		String originNodeName,
		ICommand command,
		String futureId)
	{
		sourceDestinationAgentPath.checkValidity();
		CommandAckEnvelope env = new CommandAckEnvelope(sourceDestinationAgentPath, command, getLogger(), futureId, originNodeName);
		return env;
	}

	protected ICommandEnvelope getCommandEnvelope(AgentPath sourceDestinationAgentPath, String originNodeName, ICommand command)
	{
		sourceDestinationAgentPath.checkValidity();
		ICommandEnvelope env = new CommandEnvelope(sourceDestinationAgentPath, command, getLogger(), originNodeName);
		return env;
	}

	protected CommandReplyEnvelope getCommandReplyEnvelope(
		AgentPath sourceDestinationAgentPath,
		String originNodeName,
		ICommand command,
		String futureId)
	{
		sourceDestinationAgentPath.checkValidity();
		CommandReplyEnvelope env = new CommandReplyEnvelope(sourceDestinationAgentPath, command, getLogger(), futureId, originNodeName);
		return env;
	}

	protected CommandCallbackEnvelope getCommandCallbackHandlerEnvelope(
		AgentPath sourceDestinationAgentPath,
		String originNodeName,
		ICommand command,
		String futureId)
	{
		sourceDestinationAgentPath.checkValidity();
		CommandCallbackEnvelope env = new CommandCallbackEnvelope(sourceDestinationAgentPath, command, getLogger(), futureId, originNodeName);
		return env;
	}

	/**
	 * @param command
	 * @param handler
	 */
	public void accept(ICommand command, IHistorizableCallbackHandler handler) throws RemoteException, InvalidCommandException
	{
		checkValidity(command);
		AgentPath replyToAgentPath = replyPathSameAgent();
		replyToAgentPath.checkValidity();
		String handlerId = application.getNode().getUniqueId().getValue();
		CommandCallbackEnvelope env = getCommandCallbackHandlerEnvelope(agentPath, application.getNode().getName(), command, handlerId);
		application.getObjectSpace().put(handlerId, handler);
		env.setReplyToAgentPath(replyToAgentPath);
		env.aboutToBeSent();
		env.setSendTime();
		agentDispatcher.accept(env);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.almendra.commons.janet.cas.logger.node.AbstractAgentProxy#replyPathSameAgent()
	 */
	protected AgentPath replyPathSameAgent()
	{
		AgentPath replyAgentPath = new AgentPath(agentPath);
		replyAgentPath.setNodeName(application.getNode().getName());
		return replyAgentPath;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.almendra.commons.commons.util.logging.ILoggable#registeredLogEvents()
	 */
	public List<String> registeredLogEvents()
	{
		List<String> events = new StringList();
		events.add(LOG_EVENT_SENT_COMMAND);
		events.add(LOG_EVENT_NO_ALTERNATIVE_AGENT_FOR_WORKLOAD_SHARING);
		events.add(LOG_EVENT_NO_WORKLOAD_BALANCING_POSSIBLE);
		events.add(LOG_EVENT_NO_WORLOAD_SHARING_AVAILABLE);
		events.add(LOG_EVENT_NO_OBSERVER_ON_SAME_WORKSTATION);
		return events;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.logger.node.AbstractAgentProxy#setLogger(org.almendra.commons.janet.cas.logger.Logger)
	 */
	public void setLogger(Logger logger)
	{
		super.setLogger(logger);
		// TODO - one day subscribing to events in hard coded way has to be cleaned up.
		getLogger().registerForEvents(registeredLogEvents());
		getLogger().subscribeForEvents(registeredLogEvents());
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.agents.IAgentProxy#accept(org.almendra.commons.janet.cas.scheduling.ICommandEnvelope)
	 */
	public void accept(ICommandEnvelope env) throws RemoteException, InvalidCommandException
	{
		checkValidity(env.getCommand());
		env.aboutToBeSent();
		logSentCommand(env);
		env.setSendTime();
		agentDispatcher.accept(env);
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.agents.IAgentProxy#size()
	 */
	public int size()
	{
		return 1;
	}
	
	public LocalCapabilityPath getLocalCapabilityPath()
	{
		return new LocalCapabilityPath(agentPath.getApplicationName(), agentPath.getCapabilityName());		
	}
	
	public String getCapabilityPath()
	{
		return getLocalCapabilityPath().getQualifiedPathName();		
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.agents.IAgentProxy#accept(org.almendra.commons.janet.cas.scheduling.IWorkLoadSharingCommand)
	 */
	public void accept(IWorkLoadSharingCommand sharingCommand) throws RemoteException, InvalidCommandException
	{
		IExecutorAnchor executorAnchor = (IExecutorAnchor) getNodeObjectSpace().get(IExecutorAnchor.LABEL);
		if(executorAnchor == null)
		{
			logNoLoadSharingAvailable(sharingCommand);
			sendNoWorkLoadCommand(sharingCommand);
			return;
		}
		
		if(!executorAnchor.isConnectedWithAnObserver())
		{
			logNoObserverOnSameWorkstation(sharingCommand);
			sendNoWorkLoadCommand(sharingCommand);
			return;
		}
		
		if(!existsAlternativeAgents())
		{
			logNoAlternativeForWorkloadSharingCommand(sharingCommand);
			sendNoWorkLoadCommand(sharingCommand);
			return;
		}
		
		IAgentProxy loadSharingAgent = getLoadSharingAgent();
		if(loadSharingAgent == null)
		{
			logNoLoadSharingAvailable(sharingCommand);
			sendNoWorkLoadCommand(sharingCommand);
			return;
		}
		
		ICommand cmd = executorAnchor.getWorkloadDistributionSharingCommandEnvelope(agentPath, sharingCommand, getCapabilityPath());
		loadSharingAgent.accept(cmd, ICommandEnvelope.COMMAND_TYPE_SHARING);
	}

	/**
	 * @param sharingCommand
	 */
	protected void logNoObserverOnSameWorkstation(IWorkLoadSharingCommand sharingCommand)
	{
		LogEntry entry = logger.getNewLogEntry();
		entry.setEvent(LOG_EVENT_NO_OBSERVER_ON_SAME_WORKSTATION);

		StringBuffer buffer = new StringBuffer();
		buffer.append("{No observer node found on local workstation or node not connected with local observer!"); //$NON-NLS-1$
		buffer.append(" Command " + sharingCommand.toLogString() + " sent to local agent " + agentPath.getQualifiedPathName() + "}");	//$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		
		entry.setArgument(buffer.toString());
		logger.error(entry);
	}

	/**
	 * @param sharingCommand
	 */
	protected void logNoLoadSharingAvailable(IWorkLoadCommand command)
	{
		LogEntry entry = logger.getNewLogEntry();
		entry.setEvent(LOG_EVENT_NO_WORLOAD_SHARING_AVAILABLE);

		StringBuffer buffer = new StringBuffer();
		buffer.append("{Janet.AD agent for workload sharing not found!"); //$NON-NLS-1$
		buffer.append(" Command " + command.toLogString() + " sent to local agent " + agentPath.getQualifiedPathName() + "}");	//$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		
		entry.setArgument(buffer.toString());
		logger.error(entry);
	}

	/**
	 * @param sharingCommand
	 */
	protected void logNoAlternativeForWorkloadSharingCommand(IWorkLoadSharingCommand sharingCommand)
	{
		LogEntry entry = logger.getNewLogEntry();
		entry.setEvent(LOG_EVENT_NO_ALTERNATIVE_AGENT_FOR_WORKLOAD_SHARING);

		StringBuffer buffer = new StringBuffer();
		buffer.append("{no alternative agent found for " + agentPath.getQualifiedPathName()); //$NON-NLS-1$ //$NON-NLS-2$
		buffer.append(" to accept command " + sharingCommand.toLogString() + "}");	//$NON-NLS-1$ //$NON-NLS-2$
		
		entry.setArgument(buffer.toString());
		logger.warn(entry);
	}

	/**
	 * @return
	 */
	protected IAgentProxy getLoadSharingAgent()
	{		
		IExecutorAnchor executorAnchor = (IExecutorAnchor) getNodeObjectSpace().get(IExecutorAnchor.LABEL);
		if(executorAnchor == null)
			return null;
		return executorAnchor.getNextLoadSharingAgent();
	}

	/**
	 * @param sharingCommand
	 */
	protected void sendNoWorkLoadCommand(IWorkLoadSharingCommand sharingCommand) 
	throws RemoteException, InvalidCommandException
	{
		ICommand command = sharingCommand;
		accept(command);
	}
	
	/**
	 * @param sharingCommand
	 */
	protected void sendNoWorkLoadCommand(IWorkLoadBalancingCommand balancingCommand) 
	throws RemoteException, InvalidCommandException
	{
		ICommand command = balancingCommand;
		accept(command);
	}

	protected boolean existsAlternativeAgents()
	{
		int number = getNumberOfAgentsWithCapability();
		return number > 1;
	}
	
	protected int getNumberOfAgentsWithCapability()
	{		
		return getAgentsNodeNamesWithCapability(agentPath.getCapabilityName()).size();
	}
	
	protected Set<String> getAgentsNodeNamesWithCapability(String capabilityName)
	{		
		Set<String> nodeNames = new StringSet();
		List<AgentPath> agentPaths = application.getAgentPaths(capabilityName);
		for(AgentPath agentPath : agentPaths)
			nodeNames.add(agentPath.getNodeName());
		return nodeNames;
	}

	/**
	 * @return
	 */
	public boolean isIdle()
	{
		return true;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.agents.IAgentProxy#accept(org.almendra.commons.janet.cas.scheduling.IWorkLoadBalancingCommand)
	 */
	public void accept(IWorkLoadBalancingCommand command) throws RemoteException, InvalidCommandException
	{
		IExecutorAnchor executorAnchor = (IExecutorAnchor) getNodeObjectSpace().get(IExecutorAnchor.LABEL);
		if(executorAnchor == null)
		{
			logNoLoadBalancingAvailable(command);
			sendNoWorkLoadCommand(command);
			return;
		}
		
		if(!executorAnchor.isAboveThreshold())
		{
			ICommand balancingCommand = executorAnchor.getExecutebalancingCommand(command, getCapabilityPath());
			accept(balancingCommand, ICommandEnvelope.COMMAND_TYPE_BALANCING);
			return;
		}
		
		IAgentProxy loadSharingAgent = getLoadSharingAgent();
		if(loadSharingAgent == null)
		{
			logNoLoadSharingAvailable(command);
			sendNoWorkLoadCommand(command);
			return;
		}
		
		ICommand cmd = executorAnchor.getWorkloadDistributionBalancingCommandEnvelope(agentPath, command, getCapabilityPath());
		loadSharingAgent.accept(cmd, ICommandEnvelope.COMMAND_TYPE_BALANCING);
	}
	
	/**
	 * @param command
	 */
	protected void logNoLoadBalancingAvailable(IWorkLoadBalancingCommand command)
	{
		LogEntry entry = logger.getNewLogEntry();
		entry.setEvent(LOG_EVENT_NO_WORKLOAD_BALANCING_POSSIBLE);

		StringBuffer buffer = new StringBuffer();
		buffer.append("{no workload balancing agent found! " + command.toLogString()); //$NON-NLS-1$
		buffer.append(" will be executed by agent " + agentPath.getQualifiedPathName() + "}"); //$NON-NLS-1$ //$NON-NLS-2$
		
		entry.setArgument(buffer.toString());
		logger.warn(entry);
	}

	public boolean isLocal()
	{
		try
		{
			return agentDispatcher.getName().equals(getOriginNodeName());
		}
		catch (RemoteException e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
			getLogger().exception(e);
		}
		return false;
	}

	protected IObjectSpace getNodeObjectSpace()
	{
		return application.getNodeObjectSpace();
	}

	public void accept(ICommand command, Acknowledge ack) 
	throws RemoteException, InvalidCommandException, IllegalArgumentException
	{
		checkValidity(command);
		String futureId = application.getNode().getUniqueId().getValue();
		CommandAckEnvelope env = getCommandAckEnvelope(agentPath, application.getNode().getName(), command, futureId);		
		setReplyAgentPath(ack, env);
		application.getObjectSpace().put(futureId, ack);
		env.aboutToBeSent();
		logSentCommand(env);
		env.setSendTime();
		agentDispatcher.accept(env);
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.agents.IAgentProxy#accept(org.almendra.commons.janet.cas.scheduling.IWorkLoadSharingCommand, org.almendra.commons.janet.cas.scheduling.Acknowledge)
	 */
	public void accept(IWorkLoadSharingCommand command, Acknowledge ack) throws RemoteException, InvalidCommandException
	{
		AssertUtility.assertTrue("not yet implemented", false); //$NON-NLS-1$
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.agents.IAgentProxy#accept(org.almendra.commons.janet.cas.scheduling.IWorkLoadSharingCommand, org.almendra.commons.commons.threads.FutureResult)
	 */
	public void accept(IWorkLoadSharingCommand command, FutureResult futureResult) throws IllegalArgumentException, RemoteException, InvalidCommandException
	{
		AssertUtility.assertTrue("not yet implemented", false); //$NON-NLS-1$
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.agents.IAgentProxy#accept(org.almendra.commons.janet.cas.scheduling.IWorkLoadBalancingCommand, org.almendra.commons.commons.events.IHistorizableCallbackHandler)
	 */
	public void accept(IWorkLoadBalancingCommand command, IHistorizableCallbackHandler handler) throws RemoteException, InvalidCommandException
	{
		AssertUtility.assertTrue("not yet implemented", false); //$NON-NLS-1$
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.agents.IAgentProxy#accept(org.almendra.commons.janet.cas.scheduling.IWorkLoadBalancingCommand, org.almendra.commons.janet.cas.scheduling.Acknowledge)
	 */
	public void accept(IWorkLoadBalancingCommand command, Acknowledge ack) throws RemoteException, InvalidCommandException
	{
		AssertUtility.assertTrue("not yet implemented", false); //$NON-NLS-1$
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.agents.IAgentProxy#accept(org.almendra.commons.janet.cas.scheduling.IWorkLoadBalancingCommand, org.almendra.commons.commons.threads.FutureResult)
	 */
	public void accept(IWorkLoadBalancingCommand command, FutureResult futureResult) throws IllegalArgumentException, RemoteException, InvalidCommandException
	{
		AssertUtility.assertTrue("not yet implemented", false); //$NON-NLS-1$
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.agents.IAgentProxy#accept(org.almendra.commons.janet.cas.scheduling.IWorkLoadSharingCommand, org.almendra.commons.commons.events.IHistorizableCallbackHandler)
	 */
	public void accept(IWorkLoadSharingCommand command, IHistorizableCallbackHandler handler) throws RemoteException, InvalidCommandException
	{
		AssertUtility.assertTrue("not yet implemented", false); //$NON-NLS-1$
	}
}
