/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source code or
 * in binary form must reproduce the above copyright notice, and the disclaimer
 * that follows.
 * 
 * In addition, you are required that you include in the end-user documentation
 * provided with the redistribution and/or in the software itself an
 * acknowledgement equivalent to the following: "This product is based on the
 * Janet software developed by the JANET Project
 * (http://www.almendra.org/janet/)."
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE JDOM
 * AUTHORS OR THE PROJECT CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */

package org.almendra.janet.cas.central.interpreters;

import java.util.List;

import org.almendra.commons.util.AssertUtility;
import org.almendra.commons.util.collection.StringList;
import org.almendra.commons.util.exception.InsufficientPrivilegeException;
import org.almendra.janet.cas.commands.DeregisterNodeCommand;
import org.almendra.janet.cas.commands.NodeShutdownCommand;
import org.almendra.janet.cas.node.CentralAnchor;
import org.almendra.janet.cas.node.interpreters.DeregisterNodeTrait;
import org.almendra.janet.cas.node.interpreters.NodeShutdownInterpreter;
import org.almendra.janet.cas.scheduling.CommandAccessor;


/**
 * @author Oliver
 *  
 */
public class DeregisterNodeInterpreter extends CentralInterpreter
{
	DeregisterNodeTrait deregisterNodeTrait = new DeregisterNodeTrait();

	/**
	 * Creates a new <code>DeregisterNodeInterpreter</code> object.
	 */
	public DeregisterNodeInterpreter()
	{
		super();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.almendra.commons.janet.cas.logger.schedulers.IInterpreter#execute(org.almendra.commons.janet.cas.logger.schedulers.CommandAccessor)
	 */
	public void execute(CommandAccessor cmdAccessor)
	{
		if (cmdAccessor.getCommand() instanceof DeregisterNodeCommand)
		{
			DeregisterNodeCommand cmd = (DeregisterNodeCommand) cmdAccessor.getCommand();
			
			if(deregisteringNodeIsCentral(cmdAccessor))
				deregisterCentral(cmdAccessor);								
			else
				deregisterNode(cmdAccessor, cmd.getNodeName());
			
			unbind(cmdAccessor, cmd);
		}
	}

	protected void deregisterNode(CommandAccessor cmdAccessor, String nodeName)
	{
		try
		{
			CentralAnchor anchor = (CentralAnchor) cmdAccessor.getNode().getNodeAnchor();
			boolean isCentral = CentralAnchor.NAME.equals(nodeName); 
			if(!isCentral)
			{	
				boolean objectFound = anchor.removeShutdownRequest(nodeName);
				AssertUtility.assertTrue(objectFound);
			}
			
			deregisterNodeTrait.deregisterNode(cmdAccessor);
			
			if(!isCentral)
			{	
				String nodeWaitingForShutdownPermission = anchor.getNextShutdownRequest();
				if(nodeWaitingForShutdownPermission != null)
					cmdAccessor.getSystemAgent(nodeName).accept(new NodeShutdownCommand());
			}
		}
		catch (Exception e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
			cmdAccessor.getLogger().exception(e);
		}		
	}

	protected void deregisterCentral(CommandAccessor cmdAccessor)
	{
		deregisterNodeTrait.deregisterCentral(cmdAccessor);
		unbindClusterSubobjects(cmdAccessor);
	}

	/**
	 * @param cmdAccessor
	 */
	protected void unbindClusterSubobjects(CommandAccessor cmdAccessor)
	{		
		NodeShutdownInterpreter.unbindClusterSubobjects(cmdAccessor);
	}

	protected void unbind(CommandAccessor cmdAccessor, DeregisterNodeCommand cmd)
	{
		try
		{
			deregisterNodeTrait.unbind(cmdAccessor.getSystemApplication().getCentralRegistryLocation(), cmd.getRMINodeName());
		}
		catch (Exception e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
			cmdAccessor.getLogger().exception(e);
		}
	}

	/**
	 * @param cmdAccessor
	 * @return
	 */
	protected boolean deregisteringNodeIsCentral(CommandAccessor cmdAccessor)
	{
		DeregisterNodeCommand cmd = (DeregisterNodeCommand) cmdAccessor.getCommand();
		try
		{
			return cmdAccessor.getNode().getName().equals(cmd.getNodeName());
		}
		catch (InsufficientPrivilegeException e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
			cmdAccessor.getLogger().exception(e);
			return false;
		}
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.scheduling.IInterpreter#commandNames()
	 */
	public List<String> commandNames()
	{
		return new StringList(DeregisterNodeCommand.QualifiedName);
	}

}
