/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.almendra.org/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE JANET AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.janet.cas.events;

import java.rmi.RemoteException;
import java.rmi.registry.Registry;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.NoSuchElementException;

import org.almendra.commons.events.Event;
import org.almendra.commons.events.EventRegistryCore;
import org.almendra.commons.util.StringUtility;
import org.almendra.commons.util.collection.IIdentifiableListEvaluator;
import org.almendra.commons.util.exception.PropagatedException;
import org.almendra.commons.util.inet.RMIUtility;
import org.almendra.commons.util.inet.RegistryLocation;



public class ClusterEventRegistry<EventNotifierType extends ClusterEventNotifier> extends EventRegistryCore<EventNotifierType> implements IClusterEventRegistry
{
	public static final String RMI_PREFIX = "Janet.clusterEventRegistry."; //$NON-NLS-1$
	
	protected Map<String, ILocalClusterEventRegistryRemote> localRegistriesByNodeName = new HashMap<String, ILocalClusterEventRegistryRemote>();
	
	/**
	 * Creates a new <code>ClusterEventRegistry</code> object.
	 */
	public ClusterEventRegistry()
	{
		super();
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.events.IClusterEventRegistry#register(java.lang.String)
	 */
	public void register(String localEventRegistryName, RegistryLocation registryLocation)
	{			
	    accessProtection.lock();
		try
		{			
			Registry registry = RMIUtility.getRegistry(registryLocation);
			ILocalClusterEventRegistryRemote localRegistry = (ILocalClusterEventRegistryRemote) registry.lookup(localEventRegistryName);
			String nodeName = StringUtility.remove(localEventRegistryName, LocalClusterEventRegistry.RMI_PREFIX);
			localRegistriesByNodeName.put(nodeName, localRegistry);
		}
		catch (Exception e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
		}
		finally
		{
			accessProtection.unlock();
		}				
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.events.IClusterEventRegistry#deregister(java.lang.String)
	 */
	public boolean deregister(String nodeName)
	{
	    accessProtection.lock();
		boolean found = false;
		try
		{			
			localRegistriesByNodeName.remove(nodeName);
			Collection<List<EventNotifierType>> notifiersList = registry.values();
			for(List<EventNotifierType> notifiers : notifiersList)
			{			    
			    List<EventNotifierType> copiedNotifiers = new ArrayList<EventNotifierType>(notifiers);
			    for(EventNotifierType notifier : copiedNotifiers)
			    {
			        // List<ClusterEventNotifier> wrde compilieren 
			        found |= notifier.removeHandlers(nodeName);
			    }
			}
			return found;
		}
		catch (Exception e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
		}
		finally
		{
			accessProtection.unlock();	
		}
		return found;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.events.IClusterEventRegistry#deregister(java.lang.String, java.lang.String)
	 */
	public boolean deregister(String eventName, String handlerListId) throws NoSuchElementException, IllegalArgumentException
	{
		accessProtection.lock();
		try
		{
		    List<EventNotifierType> notifiers = registry.get(eventName);
			if (notifiers == null)
				throw new NoSuchElementException("no event registered named " + eventName); //$NON-NLS-1$
			List<EventNotifierType> copiedNotifiers = new ArrayList<EventNotifierType>(notifiers);
			for(EventNotifierType notifier : copiedNotifiers)
			{
				if (notifier.hasHandlerList(handlerListId))
					return notifiers.remove(notifier);
			}
			throw new NoSuchElementException("list evaluator for event named " + eventName + " not found!"); //$NON-NLS-1$ //$NON-NLS-2$
		}
		finally
		{
		    accessProtection.unlock();
		}				
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.commons.events.EventRegistry#getEventNotifier(org.almendra.commons.commons.util.collection.IListEvaluator)
	 */
	protected EventNotifierType getEventNotifier(IIdentifiableListEvaluator<ClusterEventHandlerEnvelope> list)
	{
		return new ClusterEventNotifier(list);
	}
	
	public boolean isRegistered(String originNodeName, String eventName)
	{
	    accessProtection.lock();
	    try
	    {
	        for(ClusterEventNotifier notifier : registry.get(eventName))
			{
				if(notifier.isRegistered(originNodeName))
					return true;
			}
			return false;
	    }
		finally
		{
		    accessProtection.unlock();
		}
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.commons.events.ILocalEventRegistry#occured(org.almendra.commons.commons.events.Event)
	 */
	public void occured(String originNodeName, Event event) throws PropagatedException
	{
	    accessProtection.lock();
		try
		{			
			for(String nodeName : localRegistriesByNodeName.keySet())
			{
				if(!originNodeName.equals(nodeName))
				{
					if(isRegistered(nodeName, event.getName()))
					{	
						ILocalClusterEventRegistryRemote localRegistry = localRegistriesByNodeName.get(nodeName);
						localRegistry.occured(event);
					}
				}
			}
		}
		catch (Exception e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
			throw new PropagatedException(e);
		}		
		finally
		{
			accessProtection.unlock();
		}
		
		
	}

    /* (non-Javadoc)
     * @see org.almendra.janet.cas.events.IClusterEventRegistry#register(java.lang.String, org.almendra.commons.util.collection.IIdentifiableListEvaluator)
     */
    public void register(String eventName, IIdentifiableListEvaluator<ClusterEventHandlerEnvelope> list) throws IllegalArgumentException, RemoteException
    {
		if (!StringUtility.isValid(eventName))
			throw new IllegalArgumentException("no name specified for event"); //$NON-NLS-1$
		accessProtection.lock();
		try
		{
		    List<EventNotifierType> notifiers = registry.get(eventName);
			if (notifiers == null)
			{
				notifiers = new LinkedList<EventNotifierType>();
				registry.put(eventName, notifiers);
			}
			notifiers.add(getEventNotifier(list));
		}
		finally
		{
		    accessProtection.unlock();
		}
    }

}
