/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source code or
 * in binary form must reproduce the above copyright notice, and the disclaimer
 * that follows.
 * 
 * In addition, you are required that you include in the end-user documentation
 * provided with the redistribution and/or in the software itself an
 * acknowledgement equivalent to the following: "This product is based on the
 * Janet software developed by the JANET Project
 * (http://www.almendra.org/janet/)."
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE JDOM
 * AUTHORS OR THE PROJECT CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */

package org.almendra.janet.cas.node;

import java.rmi.AccessException;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.rmi.registry.Registry;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

import org.almendra.commons.number.Long;
import org.almendra.commons.util.AssertUtility;
import org.almendra.commons.util.collection.ListUtility;
import org.almendra.commons.util.collection.StringList;
import org.almendra.commons.util.collection.StringSet;
import org.almendra.commons.util.inet.RMIUtility;

/**
 * @author Oliver
 */
public abstract class AbstractClusterImage
{
	protected Map<String, AbstractNodeImage> nodeImages = new HashMap<String, AbstractNodeImage>();
	protected Lock accessProtection = new ReentrantLock();
	protected AbstractNodeImage localNodeImage = null;

	public AbstractClusterImage()
	{
		super();
	}

	/**
	 * @param nodeImage
	 */
	public AbstractNodeImage removeNodeImage(String nodeName)
	{
	    accessProtection.lock();
		try
		{			
			return nodeImages.remove(nodeName);
		}
		finally
		{
			accessProtection.unlock();
		}
	}

	/**
	 * @param nodeImage
	 */
	public void deregisterAllApplications(String nodeName)
	{
	    accessProtection.lock();
		try
		{			
			AbstractNodeImage nodeImage = nodeImages.get(nodeName);
			nodeImage.deregisterAllApplications();
		}
		catch (Exception e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
		}
		finally
		{
			accessProtection.unlock();
		}
	}

	public abstract AbstractNodeImage addNodeImage(IAgentDispatcher agentDispatcher, NodeLocation nodeLocation);

	/**
	 * @param path
	 * @return
	 */
	public IAgentDispatcher getAgentDispatcher(AgentPath path) throws NoSuchAgentException
	{
		AbstractNodeImage nodeImage = getNodeImage(path.getNodeName());
		if (nodeImage == null)
			throw new NoSuchAgentException(path, NoSuchAgentException.NODE_LEVEL, path.getNodeName());
		return nodeImage.getAgentDispatcher();
	}

	/**
	 * @return
	 */
	public List<String> getAllForeignNodeNames()
	{		
	    accessProtection.lock();
		try
		{			
			List<String> nodeNames = new StringList();
			for(AbstractNodeImage nodeImage : nodeImages.values())
			{
				if (nodeImage.isForeign)
					nodeNames.add(nodeImage.getNodeName());
			}
			return nodeNames;
		}
		finally
		{
			accessProtection.unlock();
		}
	}

	public AbstractNodeImage getNodeImage(String nodeName)
	{
	    accessProtection.lock();
		try
		{			
			return nodeImages.get(nodeName);
		}
		finally
		{
			accessProtection.unlock();
		}
	}

	public AbstractNodeImage getNodeImage(NodeLocation locationOrNull, String nodeName) 
	throws AccessException, RemoteException, NotBoundException
	{
	    accessProtection.lock();
		try
		{			
			AbstractNodeImage nodeImage = nodeImages.get(nodeName);
			if (nodeImage == null && locationOrNull != null)
			{
				Registry registry = RMIUtility.getRegistry(locationOrNull.getRegistryLocation());
				IAgentDispatcher dispatcher = (IAgentDispatcher) registry.lookup(Node.RMI_LABEL_PREFIX
						+ locationOrNull.getNodeName());
				nodeImage = addLocalNode(dispatcher, locationOrNull);
			}
			return nodeImage;
		}
		finally
		{
			accessProtection.unlock();
		}
	}

	/**
	 * @param string
	 * @return
	 */
	public List<String> getForeignNodeNamesWithApplication(String applicationName)
	{	
	    accessProtection.lock();
		try
		{			
			Set<String> nodeNames = new StringSet();
			for(String nodeName : nodeImages.keySet())
			{
				AbstractNodeImage nodeImage = nodeImages.get(nodeName);
				if (nodeImage.isForeign)
				{
					for(String currentApplicationName : nodeImage.getApplications().keySet())
					{						
						if (currentApplicationName.equals(applicationName))
						{
							Set<AgentPath> paths = nodeImage.getApplications().get(currentApplicationName);
							for(AgentPath path : paths)
								nodeNames.add(path.getNodeName());
						}
					}
				}
			}
			return ListUtility.toStringList(nodeNames);
		}
		finally
		{
			accessProtection.unlock();
		}
	}

	public int getAgentCount(String applicationName, String capabilityName)
	{
		List<AgentPath> agentPaths = getAgentPaths(applicationName, capabilityName);
		if (agentPaths == null)
			return 0;
		return agentPaths.size();
	}

	/**
	 * @param string
	 * @return
	 */
	public List<AgentPath> getAgentPaths(String applicationName, String capabilityName, String agentName)
	{
	    accessProtection.lock();
		try
		{			
			List<AgentPath> agentPaths = getAgentPaths(applicationName, capabilityName);
			List<AgentPath> copiedAgentPaths = new LinkedList<AgentPath>(agentPaths);
			for(AgentPath agentPath : copiedAgentPaths)
			{
				if (!agentPath.getAgentName().equals(agentName))
					agentPaths.remove(agentPath);
			}
			
			copiedAgentPaths = new LinkedList<AgentPath>(agentPaths);
			for(AgentPath agentPath : copiedAgentPaths)
			{
				if (!agentPath.getAgentName().equals(agentName))
					agentPaths.remove(agentPath);
			}
			return agentPaths;
		}
		finally
		{
			accessProtection.unlock();
		}
	}

	/**
	 * @param string
	 * @return
	 */
	public List<AgentPath> getAgentPaths(String applicationName, String capabilityName)
	{
	    accessProtection.lock();
		try
		{			
			List<AgentPath> agentPaths = new ArrayList<AgentPath>();
			for(String nodeName : nodeImages.keySet())
			{
				AbstractNodeImage nodeImage = nodeImages.get(nodeName);
				for(String currentApplicationName : nodeImage.getApplications().keySet())
				{
					if (currentApplicationName.equals(applicationName))
					{
						Set<AgentPath> paths = nodeImage.getApplications().get(currentApplicationName);
						for(AgentPath path : paths)
						{
							if (path.getCapabilityName().equals(capabilityName))
								agentPaths.add(new AgentPath(path));
						}
					}
				}
			}
			return agentPaths;
		}
		finally
		{
			accessProtection.unlock();
		}
	}

	/**
	 * @param string
	 * @return
	 */
	public List<String> getForeignNodesWithApplication(String applicationName)
	{
	    accessProtection.lock();
		try
		{			
			Set<String> nodes = new StringSet();
			for(String nodeName : nodeImages.keySet())
			{
				AbstractNodeImage nodeImage = nodeImages.get(nodeName);
				for(String currentApplicationName : nodeImage.getApplications().keySet())
				{
					if(currentApplicationName.equals(applicationName))
						nodes.add(nodeName);
				}
			}
			return ListUtility.toStringList(nodes);
		}
		finally
		{
			accessProtection.unlock();
		}
	}

	/**
	 * @return
	 */
	public List<NodeLocation> getAllForeignNodeLocations()
	{
	    accessProtection.lock();
		try
		{			
			List<NodeLocation> allNodeLocations = new ArrayList<NodeLocation>();
			for(AbstractNodeImage nodeImage : nodeImages.values())
			{
				if (nodeImage.isForeign)
					allNodeLocations.add(nodeImage.getNodeLocation());
			}
			return allNodeLocations;
		}
		finally
		{
			accessProtection.unlock();
		}
	}

	public List<AgentPath> getAllAgents(LocalAgentPath localPath)
	{
		return getAllForeignAgents(localPath, false);
	}

	public List<AgentPath> getAllForeignAgents(LocalAgentPath localPath)
	{
		return getAllForeignAgents(localPath, true);
	}

	/**
	 * @param string
	 * @param capabilityName
	 * @param agentName
	 * @return
	 */
	public List<AgentPath> getAllForeignAgents(LocalAgentPath localPath, boolean onlyForeign)
	{	
	    accessProtection.lock();
		try
		{			
			List<AgentPath> allAgentPaths = new ArrayList<AgentPath>();
			String applicationName = localPath.getApplicationName();
			String capabilityName = localPath.getCapabilityName();
			String agentName = localPath.getAgentName();
			for(AbstractNodeImage nodeImage : nodeImages.values())
			{
				boolean considerNodeImage = true;
				if (onlyForeign)
					considerNodeImage = nodeImage.isForeign;
				if (considerNodeImage && nodeImage.hasAgent(applicationName, capabilityName, agentName))
				{
					AgentPath agentPath = new AgentPath(nodeImage.getNodeName(), applicationName, capabilityName, agentName);
					allAgentPaths.add(agentPath);
				}
			}
			return allAgentPaths;
		}
		finally
		{
			accessProtection.unlock();
		}
	}

	public int getForeignNodesCount()
	{
	    accessProtection.lock();
		try
		{			
			int size = 0;
			for(AbstractNodeImage nodeImage : nodeImages.values())
			{
				if (nodeImage.isForeign)
					size++;
			}
			return size;
		}
		finally
		{
			accessProtection.unlock();
		}
	}

	public List<AgentPath> getAllForeignSystemAgentPaths(String applicationName)
	{	
	    accessProtection.lock();
		try
		{			
			List<AgentPath> systemAgents = new ArrayList<AgentPath>();
			for(AbstractNodeImage nodeImage : nodeImages.values())
			{
				if (nodeImage.isForeign && nodeImage.hasApplication(applicationName))
				{
					AgentPath agentPath = AgentPath.getSystemPath(nodeImage.getNodeName());
					systemAgents.add(agentPath);
				}
			}
			return systemAgents;
		}
		finally
		{
			accessProtection.unlock();
		}
	}

	/**
	 * @param dispatcher
	 * @param nodeLocation
	 */
	public AbstractNodeImage addLocalNode(IAgentDispatcher dispatcher, NodeLocation nodeLocation)
	{
	    accessProtection.lock();
		try
		{			
			AbstractNodeImage nodeImage = createLocalNode(dispatcher, nodeLocation);
			localNodeImage = nodeImage;
			AssertUtility.assertFalse(nodeImages.containsKey(nodeImage.getNodeName()));
			nodeImages.put(nodeImage.getNodeName(), nodeImage);
			return nodeImage;
		}
		catch (Exception e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
			return null;
		}
		finally
		{
			accessProtection.unlock();
		}
	}

	/**
	 * @param dispatcher
	 * @param nodeLocation
	 * @return
	 */
	protected abstract AbstractNodeImage createLocalNode(IAgentDispatcher dispatcher, NodeLocation nodeLocation);

	/**
	 * @return
	 */
	public AbstractNodeImage getLocalNodeImage()
	{
		return localNodeImage;
	}

	/**
	 * @param agentPath
	 * @return
	 */
	public Map<AgentPath, Long> getAgentsQueueSizes(LocalAgentPath agentPath)
	{
	    accessProtection.lock();
		try
		{			
			Map<AgentPath, Long> agentQueueSizes = new HashMap<AgentPath, Long>();
			for(String nodeName : nodeImages.keySet())
			{
				AbstractNodeImage nodeImage = nodeImages.get(nodeName);
				try
				{
					long queueSize = nodeImage.getAgentDispatcher().getQueueSize(agentPath);
					if (queueSize != -1)
						agentQueueSizes.put(new AgentPath(nodeName, agentPath), new Long(queueSize));
				}
				catch (RemoteException e)
				{
					System.out.println(e.getMessage());
					e.printStackTrace();
				}
			}
			return agentQueueSizes;
		}
		finally
		{
			accessProtection.unlock();
		}
	}

	/**
	 * @param nodeName
	 * @param capabilityName
	 * @return
	 */
	public boolean hasCapability(String nodeName, String applicationName, String capabilityName)
	{
	    accessProtection.lock();
		try
		{			
			for(String currentNodeName : nodeImages.keySet())
			{
				if (currentNodeName.equals(nodeName))
				{
					AbstractNodeImage nodeImage = nodeImages.get(nodeName);
					return nodeImage.hasCapability(applicationName, capabilityName);
				}
			}
			return false;
		}
		finally
		{
			accessProtection.unlock();
		}		
	}
	
	public boolean hasApplication(String nodeName, String applicationName)
	{
	    accessProtection.lock();
		try
		{			
			for(String currentNodeName : nodeImages.keySet())
			{
				if (currentNodeName.equals(nodeName))
				{
					AbstractNodeImage nodeImage = nodeImages.get(currentNodeName);
					return nodeImage.hasApplication(applicationName);
				}
			}
			return false;
		}
		finally
		{
			accessProtection.unlock();
		}		
	}

	/**
	 * @param path
	 * @return
	 */
	public Set<String> getAllNodeNamesWithCapability(LocalCapabilityPath path, Set<String> nodeNamesToConsider)
	{
	    accessProtection.lock();
		try
		{			
			Set<String> nodeNames = new StringSet();
			for(String nodeName : nodeNamesToConsider)
			{
				AbstractNodeImage nodeImage = nodeImages.get(nodeName);
				if (nodeImage != null && nodeImage.hasCapability(path.getApplicationName(), path.getCapabilityName()))
				{
					nodeImage.getApplications().get(path.getApplicationName());
					nodeNames.add(nodeName);
				}
			}
			return nodeNames;
		}
		finally
		{
			accessProtection.unlock();
		}
	}

	/**
	 * @param path
	 * @return
	 */
	public List<AgentPath> getAgentPaths(CapabilityPath path)
	{
	    accessProtection.lock();
		try
		{			
			List<AgentPath> agentPaths = new ArrayList<AgentPath>();
			for(String nodeName : nodeImages.keySet())
			{
				if (nodeName.equals(path.getNodeName()))
				{
					AbstractNodeImage nodeImage = nodeImages.get(nodeName);
					List<String> agentNames = nodeImage.getAgentNames(path.getApplicationName(), path.getCapabilityName());
					for(String agentName : agentNames)
					{
						agentPaths.add(new AgentPath(path.getNodeName(), path.getApplicationName(), path.getCapabilityName(), agentName));
					}
				}
			}
			return agentPaths;
		}
		finally
		{
			accessProtection.unlock();
		}
	}

	/**
	 * @param nodeName
	 * @return
	 */
	public List<LocalCapabilityPath> getAllApplicationCapabilitesOn(String nodeName)
	{
	    accessProtection.lock();
		try
		{			
			for(String currentNodeName : nodeImages.keySet())
			{
				if (currentNodeName.equals(nodeName))
				{
					AbstractNodeImage nodeImage = nodeImages.get(currentNodeName);
					return nodeImage.getAllApplicationCapabilityPaths();
				}
			}
			return new ArrayList<LocalCapabilityPath>();
		}
		finally
		{
			accessProtection.unlock();
		}		
	}

	/**
	 * @param localCapabilityPaths
	 * @param excludingNodeNames
	 * @return
	 */
	public List<LocalCapabilityPath> existingCapabilitiesOnOtherNodes(List<LocalCapabilityPath> localCapabilityPaths, List<String> excludingNodeNames)
	{
		List<LocalCapabilityPath> existingCapabilitiesOnOtherNodes = new ArrayList<LocalCapabilityPath>();
		for(LocalCapabilityPath path : localCapabilityPaths)
		{
			if (existsCapabilityOnOtherNodes(path, excludingNodeNames))
				existingCapabilitiesOnOtherNodes.add(path);
		}
		return existingCapabilitiesOnOtherNodes;
	}

	/**
	 * @param path
	 * @param excludingNodeNames
	 * @return
	 */
	protected boolean existsCapabilityOnOtherNodes(LocalCapabilityPath path, List<String> excludingNodeNames)
	{
	    accessProtection.lock();
		try
		{			
			for(String nodeName : nodeImages.keySet())
			{
				if (!excludingNodeNames.contains(nodeName))
				{
					AbstractNodeImage nodeImage = nodeImages.get(nodeName);
					if (nodeImage.getAgentCount(path.getApplicationName(), path.getCapabilityName()) > 0)
						return true;
				}
			}
			return false;
		}
		catch (Exception e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
			return false;
		}
		finally
		{
			accessProtection.unlock();
		}
	}
}