/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.almendra.org/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE JANET AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.janet.cas.node;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.almendra.commons.util.AssertUtility;
import org.almendra.commons.util.collection.ListUtility;
import org.almendra.commons.util.collection.StringList;
import org.almendra.commons.util.inet.RegistryLocation;



/**
 * @author Oliver
 *
 */
public abstract class AbstractNodeImage
{
	protected IAgentDispatcher agentDispatcher = null;
	protected Map<String, Set<AgentPath>> applications = initApplications();
	protected NodeLocation nodeLocation = null; 
	protected boolean isForeign = true;
	
	public AbstractNodeImage()
	{
		super();
	}
	
	public AbstractNodeImage(IAgentDispatcher agentDispatcher, NodeLocation nodeLocation)
	{
		super();
		this.nodeLocation = nodeLocation;
		this.agentDispatcher = agentDispatcher;
	}
	
	public AbstractNodeImage(IAgentDispatcher agentDispatcher, NodeLocation nodeLocation, boolean isForeign)
	{
		super();
		this.nodeLocation = nodeLocation;
		this.agentDispatcher = agentDispatcher;
		this.isForeign = isForeign;
	}

	protected Map<String, Set<AgentPath>> initApplications()
	{
		applications = Collections.synchronizedMap(new HashMap<String, Set<AgentPath>>());
		return applications;
	}

	/**
	 * @return Returns the agentDispatcher.
	 */
	public IAgentDispatcher getAgentDispatcher()
	{
		return agentDispatcher;
	}

	/**
	 * @return Returns the nodeName.
	 */
	public String getNodeName()
	{
		return nodeLocation.getNodeName();
	}

	/**
	 * @param path
	 */
	public void registerForeignApplicationAgent(AgentPath path)
	{
		// TODO - check why this sometimes creates an assertion violation although node isForeign
		// AssertUtility.assertTrue(isForeign);
		registerApplicationAgents(ListUtility.toListFromSingleObject(path));
	}

	/**
	 * @return Returns the applications.
	 */
	public Map<String, Set<AgentPath>> getApplications()
	{
		return applications;
	}

	/**
	 * @return Returns the nodeRegistryLocation.
	 */
	public RegistryLocation getNodeRegistryLocation()
	{
		return nodeLocation.getRegistryLocation();
	}

	/**
	 * @return Returns the nodeLocation.
	 */
	public NodeLocation getNodeLocation()
	{
		return nodeLocation;
	}

	/**
	 * @param path
	 */
	public Set<AgentPath> deregisterForeignApplication(AgentPath path)
	{
		AssertUtility.assertTrue(isForeign);
		synchronized(applications)
		{
			return applications.remove(path.getApplicationName());
		}
	}

	/**
	 * 
	 */
	public void deregisterAllApplications()
	{
		synchronized(applications)
		{
			initApplications();
		}
	}

	/**
	 * @param applicationName
	 */
	public void deregisterForeignApplication(String applicationName)
	{
		AssertUtility.assertTrue(isForeign);
		synchronized(applications)
		{
			applications.remove(applicationName);
		}
	}

	/**
	 * @param applicationName
	 * @return
	 */
	public boolean isForeignApplicationRegistered(String applicationName)
	{
		if(!isForeign)
			return false;
		synchronized(applications)
		{
			return applications.containsKey(applicationName);
		}
	}

	/**
	 * @param applicationName
	 * @param capabilityName
	 * @param agentName
	 * @return
	 */
	public boolean hasForeignAgent(String applicationName, String capabilityName, String agentName)
	{
		if(!isForeign)
			return false;
		return hasAgent(applicationName, capabilityName, agentName);
	}
	
	/**
	 * @param applicationName
	 * @param capabilityName
	 * @param agentName
	 * @return
	 */
	public boolean hasAgent(String applicationName, String capabilityName, String agentName)
	{
		synchronized(applications)
		{
			Set<AgentPath> agentPaths = applications.get(applicationName);
			if(agentPaths == null)
				return false;
			for(AgentPath path : agentPaths)
			{
				if(path.getCapabilityName().equals(capabilityName) && path.getAgentName().equals(agentName))
					return true;
			}
		}
		return false;
	}
	
	public boolean hasCapability(String applicationName, String capabilityName)
	{
		synchronized(applications)
		{
			Set<AgentPath> agentPaths = applications.get(applicationName);
			if(agentPaths == null)
				return false;
			for(AgentPath path : agentPaths)
			{
				if(path.getCapabilityName().equals(capabilityName))
					return true;
			}
		}
		return false;
	}

	/**
	 * @param capabilityName
	 * @param applicationName
	 * @return
	 */
	public boolean isForeignCapabilityRegistered(String applicationName, String capabilityName)
	{
		if(!isForeign)
			return false;
		synchronized(applications)
		{
			Set<AgentPath> agentPaths = applications.get(applicationName);
			if(agentPaths == null)
				return false;
			for(AgentPath path : agentPaths)
			{
				if(path.getCapabilityName().equals(capabilityName))
					return true;
			}
			return false;
		}
	}

	/**
	 * @return Returns the isForeign.
	 */
	public boolean isForeign()
	{
		return isForeign;
	}
	
	public boolean hasApplication(String applicationName)
	{
		return getApplications().containsKey(applicationName);
	}

	/**
	 * @param list
	 */
	public void registerApplicationAgents(List<AgentPath> newAgentPaths)
	{
		synchronized(applications)
		{
			for(AgentPath path : newAgentPaths)
			{
				String applicationName = path.getApplicationName();
				Set<AgentPath> agentPaths = applications.get(applicationName);			
				if(agentPaths == null)
				{
					agentPaths = new HashSet<AgentPath>();				
					applications.put(applicationName, agentPaths);
				}
				agentPaths.add(path);
			}						
		}
	}

	/**
	 * @param applicationName
	 */
	public void deregisterApplicationAgents(String applicationName)
	{
		synchronized(applications)
		{
			applications.remove(applicationName);
		}
	}
	
	public int getAgentCount(String applicationName, String capabilityName)
	{
		synchronized(applications)
		{
			int count = 0;
			Set<AgentPath> agentPaths = applications.get(applicationName);
			if(agentPaths == null)
				return count;
			for(AgentPath path : agentPaths)
			{
				if(path.getCapabilityName().equals(capabilityName))
					count++;
			}
			return count;
		}
	}

	/**
	 * @param localAgentPath
	 * @return
	 */
	public boolean hasAgent(LocalAgentPath localAgentPath)
	{
		return hasAgent(localAgentPath.getApplicationName(), localAgentPath.getCapabilityName(), localAgentPath.getAgentName());
	}

	/**
	 * @param string
	 * @param string2
	 * @return
	 */
	public List<String> getAgentNames(String applicationName, String capabilityName)
	{
		synchronized(applications)
		{
			List<String> agentNames = new StringList();
			Set<AgentPath> agentPaths = applications.get(applicationName);
			if(agentPaths == null)
				return null;
			for(AgentPath path : agentPaths)
			{
				if(path.getCapabilityName().equals(capabilityName))
					agentNames.add(path.getAgentName());
			}
			return agentNames;
		}
	}

	/**
	 * @return
	 */
	public List<LocalCapabilityPath> getAllApplicationCapabilityPaths()
	{
		synchronized(applications)
		{
			List<LocalCapabilityPath> localCapabilitiesList = new ArrayList<LocalCapabilityPath>();
			for(String appName : applications.keySet())
			{
				if(!appName.equals(SystemApplication.NAME))
				{
					for(AgentPath agentPath : applications.get(appName))
					{
						LocalCapabilityPath path = new LocalCapabilityPath(agentPath.getApplicationName(), agentPath.getCapabilityName());
						if(!localCapabilitiesList.contains(path))
							localCapabilitiesList.add(path);
					}					
				}
			}
			return localCapabilitiesList;
		}
	}
	
}
