/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.almendra.org/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE JANET AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.janet.cas.node;

import java.io.Serializable;
import java.util.List;

import org.almendra.commons.util.StringUtility;
import org.almendra.commons.util.collection.StringList;


/**
 * @author Oliver
 *
 */
public class LocalCapabilityPath implements IPath, Serializable
{
	private static final long serialVersionUID = 2084687159936914657L;
	protected String capabilityName = null;
	protected String applicationName = null;
	
	/**
	 * Creates a new <code>LocalCapabilityPath</code> object.
	 */
	protected LocalCapabilityPath()
	{
		super();
	}
	
	public int hashCode() 
	{
		return getQualifiedPathName().hashCode();
	}

	/**
	 * Creates a new <code>LocalCapabilityPath</code> object.
	 */
	public LocalCapabilityPath(String applicationName, String capabilityName)
	{
		super();
		init(applicationName, capabilityName);
	}
	
	/**
	 * Creates a new <code>LocalCapabilityPath</code> object.
	 */
	public LocalCapabilityPath(String capabilityPath)
	{
		super();
		int index = capabilityPath.indexOf("."); //$NON-NLS-1$
		if(index == -1)
			throw new IllegalArgumentException("invalid capability path: " + capabilityPath); //$NON-NLS-1$
		String applicationName = capabilityPath.substring(0, index);
		String capabilityName = capabilityPath.substring(index + 1, capabilityPath.length());
		init(applicationName, capabilityName);
	}
	
	public static boolean containsApplication(List<AgentPath> agentPaths, String applicationName)
	{
		for(AgentPath agentPath : agentPaths)
		{
			if(agentPath.getApplicationName().equals(applicationName))
				return true;
		}
		return false;
	}
	
	public static boolean containsCapability(List<AgentPath> agentPaths, String capabilityName)
	{
		for(AgentPath agentPath : agentPaths)
		{
			if(agentPath.getCapabilityName().equals(capabilityName))
				return true;
		}
		return false;
	}
	
	public static List<String> getQualifiedPathNames(List<? extends IPath> localCapabilityPaths)
	{
		List<String> names = new StringList();
		for(IPath path : localCapabilityPaths)
			names.add(path.getQualifiedPathName());
		return names;
	}
	
	protected void init(String applicationName, String capabilityName)
	{
		checkValidity(applicationName, capabilityName);
		this.capabilityName = capabilityName;
		this.applicationName = applicationName;
	}

	public void checkValidity(String appName, String capabilityName)
	{
		if (!StringUtility.isValid(appName))
			throw new IllegalArgumentException("illegal application name " + appName); //$NON-NLS-1$

		if (!StringUtility.isValid(capabilityName))
			throw new IllegalArgumentException("illegal capability name " + capabilityName); //$NON-NLS-1$
	}
	
	/**
	 * @return Returns the applicationName.
	 */
	public String getApplicationName()
	{
		return applicationName;
	}

	/**
	 * @return Returns the capabilityName.
	 */
	public String getCapabilityName()
	{
		return capabilityName;
	}

	/**
	 * @param applicationName The applicationName to set.
	 */
	public void setApplicationName(String applicationName)
	{
		this.applicationName = applicationName;
	}

	/**
	 * @param capabilityName The capabilityName to set.
	 */
	public void setCapabilityName(String capabilityName)
	{
		this.capabilityName = capabilityName;
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(Object arg)
	{
		if(this == arg)
			return true;
		if(arg instanceof LocalCapabilityPath)
		{
			LocalCapabilityPath path = (LocalCapabilityPath) arg;
			return sameAttributeValues(path);
		}
		return false;
	}
	
	/**
	 * @param path
	 * @return
	 */
	protected boolean sameAttributeValues(LocalCapabilityPath path)
	{
		if(!capabilityName.equals(path.getCapabilityName()))
			return false;
		if(!applicationName.equals(path.getApplicationName()))
			return false;
		return true;
	}
	
	/**
	 * @return
	 */
	public String getQualifiedPathName()
	{
		return getApplicationName() + "." + getCapabilityName(); //$NON-NLS-1$
	}
}
