/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source code or
 * in binary form must reproduce the above copyright notice, and the disclaimer
 * that follows.
 * 
 * In addition, you are required that you include in the end-user documentation
 * provided with the redistribution and/or in the software itself an
 * acknowledgement equivalent to the following: "This product is based on the
 * Janet software developed by the JANET Project
 * (http://www.almendra.org/janet/)."
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE JDOM
 * AUTHORS OR THE PROJECT CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */

package org.almendra.janet.cas.node;

import java.rmi.RemoteException;
import java.util.List;
import java.util.NoSuchElementException;

import org.almendra.commons.events.IVetoableEventRegistry;
import org.almendra.commons.util.StringUtility;
import org.almendra.commons.util.exception.AlreadyRegisteredException;
import org.almendra.commons.util.exception.InsufficientPrivilegeException;
import org.almendra.commons.util.logging.LoggerAccessor;
import org.almendra.commons.util.swing.BillboardPanelController;
import org.almendra.commons.util.swing.TextAreaPanelController;
import org.almendra.janet.cas.events.ILocalClusterEventRegistry;
import org.almendra.janet.cas.spaces.IObjectSpace;
import org.almendra.janet.cas.spaces.ISerializableObjectSpace;


/**
 * @author Oliver Plohmann
 */
public class NodeAccessor
{	
	protected Node node = null;
	
	protected NodeAccessor()
	{
		super();
	}

	public NodeAccessor(Node node)
	{
		super();
		this.node = node;
	}

	public ApplicationAccessor registerApplication(Application app)
	throws IllegalArgumentException, RemoteException, AlreadyRegisteredException
	{		
		registerApplication(app, null);
		return new ApplicationAccessor(app);
	}

	/**
	 * Register the application in the node's application registry, create all
	 * agents, and start their schedulers. Send a notifications to all nodes in
	 * the cluster that a new application has been registered at this node.
	 * 
	 * @param app
	 * @param nodeNames
	 *            names of all nodes where to register, all foreign nodes if
	 *            null.
	 * @return @throws
	 *         AlreadyRegisteredException
	 * @throws IllegalArgumentException
	 */
	public ApplicationAccessor registerApplication(Application app, List<String> nodeNames)
	throws AlreadyRegisteredException, IllegalArgumentException, RemoteException
	{
		node.registerApplication(app, nodeNames);
		return new ApplicationAccessor(app);
	}
	
	public BillboardPanelController getBillboard()
	{
		return node.getBillboard();
	}
	
	public TextAreaPanelController getFreeTextArea()
	{
		return node.getFreeTextArea();
	}

	/**
	 * @param name
	 * @return @throws
	 *         NoSuchElementException
	 */
	public void deregisterApplication(String name) throws NoSuchElementException
	{
		node.deregisterApplication(name);
	}

	public IAgentDispatcher getAgentDispatcher()
	{
		// TODO - must be removed: user must not have access to the agent
		// dispatcher but use an agent proxy
		return node.getAgentDispatcher();
	}

	/**
	 * @param applicationName
	 * @param capabilityName
	 * @param agentName
	 * @return
	 */
	public AgentPath getAgentPath(String applicationName, String capabilityName, String agentName)
	{
		return node.getAgentPath(applicationName, capabilityName, agentName);
	}

	/**
	 * @param applicationName
	 * @return
	 */
	public ApplicationAccessor getApplication(String applicationName)
	throws NoSuchElementException, RemoteException, InsufficientPrivilegeException
	{	
		if(!StringUtility.isValid(applicationName))
			throw new IllegalArgumentException("illegal application name: " + StringUtility.print(applicationName)); //$NON-NLS-1$
		
		if(applicationName.equals(SystemApplication.NAME))
			throw new InsufficientPrivilegeException("no permission to request system application"); //$NON-NLS-1$
		
		// cast legal here as it can be assumed that it can't be of kind SystemApplication
		Application app = (Application) node.getApplication(applicationName);
		
		if(app == null)
			throw new NoSuchElementException("no application with name " + StringUtility.print(applicationName) + " exists!"); //$NON-NLS-1$ //$NON-NLS-2$		
		
		node.propagateRegisteredApplication(app, null);
		return new ApplicationAccessor(app);
	}

	/**
	 * @param name
	 * @param app
	 * @return
	 * @throws NoSuchElementException
	 */
	public ILocalClusterEventRegistry getClusterEventRegistry(String name) throws NoSuchElementException, RemoteException
	{
		return node.getClusterEventRegistry(name);
	}

	/**
	 * @return
	 * @throws NoSuchElementException
	 */
	public ISerializableObjectSpace getClusterObjectSpace() throws NoSuchElementException
	{
		return node.getClusterObjectSpace();
	}

	/**
	 * @param name
	 * @return
	 * @throws NoSuchElementException
	 */
	public ISerializableObjectSpace getClusterObjectSpace(String name) throws NoSuchElementException
	{
		return node.getClusterObjectSpace(name);
	}

	/**
	 * @return
	 */
	public LoggerAccessor getLogAccessor()
	{
		return node.getLogAccessor();
	}

	/**
	 * @return
	 */
	public String getName()
	{
		return node.getName();
	}

	/**
	 * @return
	 */
	public IObjectSpace getNodeObjectSpace()
	{
		return node.getNodeObjectSpace();
	}

	/**
	 * @return
	 */
	public String getRMIName()
	{
		return node.getRMIName();
	}

	protected SystemApplication getSystemApplication()
	{
		return node.getSystemApplication();
	}

	public boolean isApplicationRegistered(String applicationName)
	{
		return node.applicationRegistry.isRegistered(applicationName);
	}

	public boolean isForeignApplicationRegistered(String applicationName, String nodeName)
	{
		return node.isForeignApplicationRegistered(applicationName, nodeName);
	}

	public boolean isForeignCapabilityRegistered(CapabilityPath path)
	{
		return node.isForeignCapabilityRegistered(path);
	}

	/**
	 * @return
	 */
	public boolean isIdle()
	{
		return node.isIdle();
	}

	/**
	 * Shuts down the node. Strictly speaking would require very high permission and therefore
	 * NodeAccessor should not allow the user to shut down a node. But somewhen some user must be 
	 * able to shut down node. Left to the responsibility of the Java VM owner not to call <code>shutdown</code>
	 * mistakenly. 
	 */
	public boolean shutdown()
	{
		return node.shutdown();
	}

	public void start()
	{
		node.start();
	}

	public void stop()
	{
		node.stop();
	}

	/**
	 * @return
	 */
	public IVetoableEventRegistry getVetoableEventRegistry()
	{
		return node.getVetoableEventRegistry();
	}

	/**
	 * @return
	 */
	public List<String> getAllNodeNames()
	{
		return getSystemApplication().getAllNodeNames();
	}

	/**
	 * @param string
	 */
	public boolean hasApplication(String nodeName, String applicationName)
	{
		return getSystemApplication().hasApplication(nodeName, applicationName);
	}
}
