/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source code or
 * in binary form must reproduce the above copyright notice, and the disclaimer
 * that follows.
 * 
 * In addition, you are required that you include in the end-user documentation
 * provided with the redistribution and/or in the software itself an
 * acknowledgement equivalent to the following: "This product is based on the
 * Janet software developed by the JANET Project
 * (http://www.almendra.org/janet/)."
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE JDOM
 * AUTHORS OR THE PROJECT CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */

package org.almendra.janet.cas.node;

import java.rmi.registry.Registry;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.almendra.commons.util.BooleanUtility;
import org.almendra.commons.util.StringUtility;
import org.almendra.commons.util.collection.StringList;
import org.almendra.commons.util.inet.InetUtility;
import org.almendra.commons.util.inet.RegistryLocation;
import org.almendra.commons.util.xml.JDOMUtility;
import org.almendra.commons.util.xml.XMLParseException;
import org.almendra.janet.cas.events.ClusterEventRegistry;
import org.almendra.janet.cas.events.LocalClusterEventRegistry;
import org.almendra.janet.cas.spaces.ClusterSpace;
import org.jdom.Element;


/**
 * @author Oliver
 *  
 */
public class NodeDescriptor extends Descriptor
{
	public static final List<String> SUPPORTED_VERSIONS = GetSupportedVersions();

	public static final String FILE_NAME = "NodeDescriptor.xml"; //$NON-NLS-1$
	public static final String LABEL = "NodeDescriptor"; //$NON-NLS-1$
	
	public static final String TAG_REGISTRY = "registry"; //$NON-NLS-1$
	public static final String TAG_VERSION = "version"; //$NON-NLS-1$
	public static final String TAG_SHOW_GUI = "showGUI"; //$NON-NLS-1$
	public static final String TAG_EXIT_VM_ON_NODE_SHUTDOWN = "exitVMOnNodeShutdown"; //$NON-NLS-1$
	public static final String TAG_CLEAR_TEXT_TAB_AFTER_LINES = "clearTextTabAfterLines"; //$NON-NLS-1$
	public static final String TAG_VERSION_ATTRIBUTE_NAME = "name"; //$NON-NLS-1$
	public static final String TAG_REGISTRY_ATTRIBUTE_CENTRAL_HOSTNAME = "centralHostname"; //$NON-NLS-1$
	public static final String TAG_REGISTRY_ATTRIBUTE_CENTRAL_PORT = "centralPort"; //$NON-NLS-1$
	public static final String TAG_REGISTRY_ATTRIBUTE_LOCAL_PORT = "localPort"; //$NON-NLS-1$
	public static final String TAG_ARBITRATION = "arbitration"; //$NON-NLS-1$
	public static final String TAG_ARBITRATION_ATTRIBUTE_ENABLE_APPLICATION_ARBITRATOR = "enableApplicationArbitrator"; //$NON-NLS-1$
	public static final String TAG_ARBITRATION_ATTRIBUTE_ENABLE_SYSTEM_ARBITRATOR = "enableSystemArbitrator"; //$NON-NLS-1$
	
	public static final String DESCRIPTOR_BASE_DIR = "DescriptorBaseDir"; //$NON-NLS-1$

	protected SystemApplicationDescriptor systemApplicationDescriptor = null;
	protected List<ApplicationDescriptor> applicationDescriptors = new LinkedList<ApplicationDescriptor>();
	protected List<ClusterSpaceDescriptor> clusterSpaceDescriptors = new LinkedList<ClusterSpaceDescriptor>();
	protected List<ClusterEventRegistryDescriptor> clusterEventRegistryDescriptors = new LinkedList<ClusterEventRegistryDescriptor>();
	protected String versionName = null;
	protected RegistryLocation centralRegistryLocation = null;
	protected RegistryLocation localRegistryLocation = null;
	protected LocalLoggerDescriptor localLoggerDescriptor = new LocalLoggerDescriptor();
	protected boolean showGUI = false;
	protected boolean exitVMOnNodeShutdown = false;	
	protected int clearTextTabAfterLines = -1;
	protected boolean enableApplicationArbitrator = true;
	protected boolean enableSystemArbitrator = true;

	public NodeDescriptor()
	{
		super();
	}

	/**
	 * @return
	 */
	private static List<String> GetSupportedVersions()
	{
		List<String> supportedVersions = new StringList();
		supportedVersions.add("0.1"); //$NON-NLS-1$
		return supportedVersions;
	}

	/**
	 * @return Returns the systemApplicationDescriptor.
	 */
	public SystemApplicationDescriptor getSystemApplicationDescriptor()
	{
		return systemApplicationDescriptor;
	}

	/**
	 * @param systemApplicationDescriptor
	 *            The systemApplicationDescriptor to set.
	 */
	public void setSystemApplicationDescriptor(SystemApplicationDescriptor systemApplicationDescriptor)
	{
		this.systemApplicationDescriptor = systemApplicationDescriptor;
	}

	public static NodeDescriptor create(Element element) throws XMLParseException
	{
		NodeDescriptor nodeDesc = new NodeDescriptor();
		nodeDesc.fill(element);				
		return nodeDesc;
	}

	/**
	 * @param element
	 */
	protected void fill(Element element) throws XMLParseException
	{
		getLocalLoggerDescriptor().fill(element);
		fillArbitration(element);
		fillClusterSpaces(element);
		fillClusterEventRegistries(element);
		checkVersion(element);
		readShowGUI(element);
		readExitVMOnNodeShutdown(element);
		readClearTextTabAfterLines(element);
		fillRegistry(element);

		Element applicationsNode = JDOMUtility.getChild(element, ApplicationDescriptor.XMLParentTag);

		Element systemApplicationNode = JDOMUtility.getChild(applicationsNode, SystemApplicationDescriptor.XMLNodeTag);
		SystemApplicationDescriptor sysAppDesc = SystemApplicationDescriptor.createSysAppDesc(systemApplicationNode);
		setSystemApplicationDescriptor(sysAppDesc);

		List applications = JDOMUtility.getChildren(applicationsNode, ApplicationDescriptor.XMLNodeTag, false);
		List<ApplicationDescriptor> appDescs = new LinkedList<ApplicationDescriptor>();

		for (int i = 0; i < applications.size(); i++)
		{
			Element applicationNode = (Element) applications.get(i);
			ApplicationDescriptor desc = ApplicationDescriptor.createAppDesc(applicationNode);
			appDescs.add(desc);
		}

		setApplicationDescriptors(appDescs);		
	}

	/**
	 * @param element
	 */
	protected void fillArbitration(Element element) throws XMLParseException
	{
		Element arbitrationElement = element.getChild(TAG_ARBITRATION);
		if(arbitrationElement == null)
			return;
		
		String enableApplicationArbitratorString = arbitrationElement.getAttributeValue(TAG_ARBITRATION_ATTRIBUTE_ENABLE_APPLICATION_ARBITRATOR);
		int booleanCode = BooleanUtility.getBooleanCode(enableApplicationArbitratorString);
		if(booleanCode == -1)
			throw new XMLParseException("invalid parameter: " + enableApplicationArbitratorString + "!"); //$NON-NLS-1$ //$NON-NLS-2$
		enableApplicationArbitrator = booleanCode == BooleanUtility.TRUE;
		
		String enableSystemArbitratorString = arbitrationElement.getAttributeValue(TAG_ARBITRATION_ATTRIBUTE_ENABLE_SYSTEM_ARBITRATOR);
		booleanCode = BooleanUtility.getBooleanCode(enableSystemArbitratorString);
		if(booleanCode == -1)
			throw new XMLParseException("invalid parameter: " + enableSystemArbitratorString + "!"); //$NON-NLS-1$ //$NON-NLS-2$
		enableSystemArbitrator = booleanCode == BooleanUtility.TRUE;
	}

	/**
	 * @param element
	 */
	protected void readClearTextTabAfterLines(Element element) throws XMLParseException
	{
		String clearTextTabAfterLinesString = element.getAttributeValue(TAG_CLEAR_TEXT_TAB_AFTER_LINES);
		if (clearTextTabAfterLinesString == null)
			return;
		Integer clearTextTabAfterLinesInt = StringUtility.toInteger(clearTextTabAfterLinesString);
		if(clearTextTabAfterLinesInt == null)
			throw new XMLParseException("illegal value for " + TAG_CLEAR_TEXT_TAB_AFTER_LINES + ": " + clearTextTabAfterLinesString); //$NON-NLS-1$ //$NON-NLS-2$
		clearTextTabAfterLines = clearTextTabAfterLinesInt.intValue();
		if(clearTextTabAfterLines <= 0)
			clearTextTabAfterLines = -1;
		
	}

	/**
	 * @param element
	 */
	private void readExitVMOnNodeShutdown(Element element) throws XMLParseException
	{
		String exitVMOnNodeShutdownString = element.getAttributeValue(TAG_EXIT_VM_ON_NODE_SHUTDOWN);
		if (exitVMOnNodeShutdownString == null)
			return;
		exitVMOnNodeShutdownString = exitVMOnNodeShutdownString.toLowerCase();
		if (!exitVMOnNodeShutdownString.equals("true") && !exitVMOnNodeShutdownString.equals("false")) //$NON-NLS-1$ //$NON-NLS-2$
			throw new XMLParseException("illegal value for " + TAG_EXIT_VM_ON_NODE_SHUTDOWN + ": " + exitVMOnNodeShutdownString); //$NON-NLS-1$ //$NON-NLS-2$
		exitVMOnNodeShutdown = Boolean.valueOf(exitVMOnNodeShutdownString).booleanValue();
	}

	/**
	 * @param element
	 */
	protected void fillClusterSpaces(Element element) throws XMLParseException
	{
		Element node = JDOMUtility.getChild(element, ClusterSpaceDescriptor.XMLParentTag, false);
		if(node == null) // no central objectSpace defined for this cluster
			return;
		
		List<Element> clusterSpaces = JDOMUtility.getChildren(node, ClusterSpaceDescriptor.XMLNodeTag, false);

		for (int i = 0; i < clusterSpaces.size(); i++)
		{
			Element clusterSpaceNode = clusterSpaces.get(i);
			ClusterSpaceDescriptor desc = ClusterSpaceDescriptor.create(clusterSpaceNode);
			clusterSpaceDescriptors.add(desc);
		}		
	}
	
	/**
	 * @param element
	 */
	protected void fillClusterEventRegistries(Element element) throws XMLParseException
	{
		Element node = JDOMUtility.getChild(element, ClusterEventRegistryDescriptor.XMLParentTag, false);
		if(node == null) // no central objectSpace defined for this cluster
			return;
		
		List clusterEventRegistries = JDOMUtility.getChildren(node, ClusterEventRegistryDescriptor.XMLNodeTag, false);

		for (int i = 0; i < clusterEventRegistries.size(); i++)
		{
			Element clusterEventRegistryNode = (Element) clusterEventRegistries.get(i);
			ClusterEventRegistryDescriptor desc = ClusterEventRegistryDescriptor.create(clusterEventRegistryNode);
			clusterEventRegistryDescriptors.add(desc);
		}		
	}

	/**
	 * @param element
	 */
	protected void readShowGUI(Element element) throws XMLParseException
	{
		String showGUIString = element.getAttributeValue(TAG_SHOW_GUI);
		if (showGUIString == null)
			return;
		showGUIString = showGUIString.toLowerCase();
		if (!showGUIString.equals("true") && !showGUIString.equals("false")) //$NON-NLS-1$ //$NON-NLS-2$
			throw new XMLParseException("illegal value for " + TAG_SHOW_GUI + ": " + showGUIString); //$NON-NLS-1$ //$NON-NLS-2$
		showGUI = Boolean.valueOf(showGUIString).booleanValue();
	}

	/**
	 * @param element
	 */
	protected void checkVersion(Element element) throws XMLParseException
	{
		versionName = element.getAttributeValue(TAG_VERSION);
		if (versionName == null)
			throw new XMLParseException("version stamp missing!"); //$NON-NLS-1$

		if (!SUPPORTED_VERSIONS.contains(versionName))
			throw new XMLParseException("version name " + versionName + " not supported!"); //$NON-NLS-1$ //$NON-NLS-2$
	}

	/**
	 * @param element
	 */
	protected void fillRegistry(Element element) throws XMLParseException
	{
		Element registryElement = element.getChild(TAG_REGISTRY);

		if (registryElement == null)
			throw new XMLParseException("no registry tag found!"); //$NON-NLS-1$

		String portString = registryElement.getAttributeValue(TAG_REGISTRY_ATTRIBUTE_LOCAL_PORT);
		if (portString != null)
		{
			try
			{
				int localPort = Integer.parseInt(portString);
				localRegistryLocation = new RegistryLocation(localPort);
			}
			catch (NumberFormatException ex)
			{
				throw new XMLParseException("invalid local port registry number: " + portString + "!"); //$NON-NLS-1$ //$NON-NLS-2$
			}
		}
		else
		{
			localRegistryLocation = new RegistryLocation(InetUtility.getHostName());
		}

		String centralHostName = registryElement.getAttributeValue(TAG_REGISTRY_ATTRIBUTE_CENTRAL_HOSTNAME);
		if(centralHostName != null && centralHostName.toLowerCase().equals(InetUtility.LOCALHOST))
			centralHostName = InetUtility.getHostName();
		if (centralHostName == null)
			throw new XMLParseException("no registry central hostname found!"); //$NON-NLS-1$
		
		int centralPort = Registry.REGISTRY_PORT;
		portString = registryElement.getAttributeValue(TAG_REGISTRY_ATTRIBUTE_CENTRAL_PORT);

		if (portString != null)
		{
			try
			{
				centralPort = Integer.parseInt(portString);
			}
			catch (NumberFormatException ex)
			{
				throw new XMLParseException("invalid central port registry number: " + portString + "!"); //$NON-NLS-1$ //$NON-NLS-2$
			}
		}

		centralRegistryLocation = new RegistryLocation(centralHostName, centralPort);
	}

	/**
	 * @return
	 */
	public SystemApplication getSystemApplication() throws DescriptorParseException
	{
		return getSystemApplicationDescriptor().toSystemApplication();
	}

	/**
	 * @return Returns the applicationDescriptors.
	 */
	public List<ApplicationDescriptor> getApplicationDescriptors()
	{
		return applicationDescriptors;
	}

	/**
	 * @param applicationDescriptors
	 *            The applicationDescriptors to set.
	 */
	public void setApplicationDescriptors(List<ApplicationDescriptor> applicationDescriptors)
	{
		this.applicationDescriptors.addAll(applicationDescriptors);
	}

	/**
	 *  
	 */
	public List<Application> getApplications() throws DescriptorParseException
	{
		List<Application> apps = new ArrayList<Application>();
		for(ApplicationDescriptor appDesc : getApplicationDescriptors())
			apps.add(appDesc.toApplication());
		return apps;
	}

	/**
	 * @return Returns the centralRegistryLocation.
	 */
	public RegistryLocation getCentralRegistryLocation()
	{
		return centralRegistryLocation;
	}

	/**
	 * @return Returns the localRegistryLocation.
	 */
	public RegistryLocation getLocalRegistryLocation()
	{
		return localRegistryLocation;
	}

	/**
	 * @return Returns the localLoggerDescriptor.
	 */
	public LocalLoggerDescriptor getLocalLoggerDescriptor()
	{
		return localLoggerDescriptor;
	}

	/**
	 * @return Returns the showGUI.
	 */
	public boolean isShowGUI()
	{
		return showGUI;
	}

	/**
	 * @param showGUI
	 *            The showGUI to set.
	 */
	public void setShowGUI(boolean showGUI)
	{
		this.showGUI = showGUI;
	}

	/**
	 * @return Returns the versionName.
	 */
	public String getVersionName()
	{
		return versionName;
	}

	/**
	 * @return Returns the clusterSpaceDescriptors.
	 */
	public List<ClusterSpaceDescriptor> getClusterSpaceDescriptors()
	{
		return clusterSpaceDescriptors;
	}

	/**
	 * @return Returns the clusterEventRegistryDescriptors.
	 */
	public List<ClusterEventRegistryDescriptor> getClusterEventRegistryDescriptors()
	{
		return clusterEventRegistryDescriptors;
	}

	/**
	 * @param clusterEventRegistryDescriptors The clusterEventRegistryDescriptors to set.
	 */
	public void setClusterEventRegistryDescriptors(List<ClusterEventRegistryDescriptor> clusterEventRegistryDescriptors)
	{
		this.clusterEventRegistryDescriptors = clusterEventRegistryDescriptors;
	}

	/**
	 * @return Returns the exitVMOnNodeShutdown.
	 */
	public boolean isExitVMOnNodeShutdown()
	{
		return exitVMOnNodeShutdown;
	}

	/**
	 * @param exitVMOnNodeShutdown The exitVMOnNodeShutdown to set.
	 */
	public void setExitVMOnNodeShutdown(boolean exitVMOnNodeShutdown)
	{
		this.exitVMOnNodeShutdown = exitVMOnNodeShutdown;
	}
	
	public static String[] getClusterSubobjectPrefixes()
	{
		String[] prefixes = new String[3];
		prefixes[0] = ClusterSpace.RMI_PREFIX;
		prefixes[1] = ClusterEventRegistry.RMI_PREFIX;
		prefixes[2] = LocalClusterEventRegistry.RMI_PREFIX;
		return prefixes;
	}

	/**
	 * @param values
	 */
	public void visit(Map<String, String> values)
	{
		systemApplicationDescriptor.visit(values);
		for(ApplicationDescriptor desc : applicationDescriptors)
			desc.visit(values);
	}

	/**
	 * @return Returns the clearTextTabAfterLines.
	 */
	public int getClearTextTabAfterLines()
	{
		return clearTextTabAfterLines;
	}
	/**
	 * @return Returns the enableApplicationArbitrator.
	 */
	public boolean isEnableApplicationArbitrator()
	{
		return enableApplicationArbitrator;
	}
	/**
	 * @return Returns the enableSystemArbitrator.
	 */
	public boolean isEnableSystemArbitrator()
	{
		return enableSystemArbitrator;
	}
}
