/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source code or
 * in binary form must reproduce the above copyright notice, and the disclaimer
 * that follows.
 * 
 * In addition, you are required that you include in the end-user documentation
 * provided with the redistribution and/or in the software itself an
 * acknowledgement equivalent to the following: "This product is based on the
 * Janet software developed by the JANET Project
 * (http://www.almendra.org/janet/)."
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE JDOM
 * AUTHORS OR THE PROJECT CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */

package org.almendra.janet.cas.node.interpreters;

import java.rmi.AccessException;
import java.rmi.AlreadyBoundException;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.rmi.registry.Registry;
import java.util.LinkedList;
import java.util.List;

import org.almendra.commons.util.StringUtility;
import org.almendra.commons.util.collection.StringList;
import org.almendra.commons.util.inet.InetUtility;
import org.almendra.commons.util.inet.RMIUtility;
import org.almendra.commons.util.inet.RegistryLocation;
import org.almendra.janet.cas.commands.NodeStartedCommand;
import org.almendra.janet.cas.commands.RegisterNodeCommand;
import org.almendra.janet.cas.node.CentralAnchor;
import org.almendra.janet.cas.node.IAgentDispatcher;
import org.almendra.janet.cas.node.InvalidCommandException;
import org.almendra.janet.cas.node.Node;
import org.almendra.janet.cas.node.NodeAnchor;
import org.almendra.janet.cas.node.NodeLocation;
import org.almendra.janet.cas.node.NodeStartupException;
import org.almendra.janet.cas.node.SystemApplication;
import org.almendra.janet.cas.scheduling.CommandAccessor;
import org.almendra.janet.cas.scheduling.IInterpreter;
import org.almendra.janet.cas.spaces.IObjectSpace;


/**
 * @author Oliver
 *  
 */
public class NodeStartedInterpreter extends NodeInterpreter implements IInterpreter
{
	public static final String ViewTitlePrefix = "Janet.node."; //$NON-NLS-1$
	protected NodeStartedTrait nodeStartedTrait = new NodeStartedTrait();

	public NodeStartedInterpreter()
	{
		super();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.almendra.commons.cas.node.IInterpreter#execute(org.almendra.commons.cas.node.ICommand)
	 */
	public void execute(CommandAccessor cmdAccessor)
	{
		if (cmdAccessor.getCommand() instanceof NodeStartedCommand)
		{
			IObjectSpace os = cmdAccessor.getObjectSpace();
			NodeStartedCommand command = (NodeStartedCommand) cmdAccessor.getCommand();

			try
			{
				if (!checkCentralExists(command.getCentralRegistryLocation()))
				{
					StringBuffer msg = NodeStartupException.getMessageHeader("no central found"); //$NON-NLS-1$
					command.getFuture().setException(new NodeStartupException(msg.toString()));
					command.getFuture().signal();
					return;
				}
			}
			catch (AccessException ex)
			{				
				System.out.println(ex.getMessage());
				ex.printStackTrace();
				cmdAccessor.log(ex);
				StringBuffer msg = NodeStartupException.getMessageHeader("cannot access central"); //$NON-NLS-1$
				msg.append(ex.getMessage());
				command.getFuture().setException(new NodeStartupException(msg.toString(), ex.getCause()));
				command.getFuture().signal();
				return;
			}
			catch (RemoteException ex)
			{
				System.out.println("\nNode cannot connect with central. Make sure that central has been started up!\n"); //$NON-NLS-1$
				System.out.println(ex.getMessage());
				ex.printStackTrace();
				cmdAccessor.log(ex);				
				StringBuffer msg = NodeStartupException.getMessageHeader("cannot connect with central"); //$NON-NLS-1$
				msg.append(ex.getMessage());
				command.getFuture().setException(new NodeStartupException(msg.toString(), ex.getCause()));
				command.getFuture().signal();
				return;
			}

			installNodeAnchor(os, command);
			
			try
			{
				bind(command, os);
			}
			catch (RemoteException ex)
			{
				System.out.println(ex.getMessage());
				ex.printStackTrace();
				cmdAccessor.log(ex);
				command.getFuture().setException(ex);
				command.getFuture().signal();
				return;
			}
			
			try
			{
				registerWithCentral(cmdAccessor);
			}
			catch (Exception ex)
			{
				System.out.println(ex.getMessage() + ": cannot register with central"); //$NON-NLS-1$
				ex.printStackTrace();
				cmdAccessor.log(ex);
				command.getFuture().setException(ex);
			}
			
			try
			{
				nodeStartedTrait.bindClusterSubobjects(cmdAccessor.getNode(), command);
			}
			catch (Exception ex)
			{
				System.out.println(ex.getMessage() + ": cannot bind cluster spaces"); //$NON-NLS-1$
				ex.printStackTrace();
				cmdAccessor.log(ex);
				command.getFuture().setException(ex);
			}
	
			// TODO - do not start arbitrator until finally registered !!!
			
			nodeStartedTrait.showGUI(command);
			command.getFuture().signal();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.almendra.commons.cas.commands.NodeStartedInterpreter#getNewNodeName()
	 */
	protected String getNewNodeName(RegistryLocation location) throws AccessException, RemoteException
	{
		List<Integer> existingPostfixesCounters = new LinkedList<Integer>();
		String newNamePrefix = InetUtility.getHostName();
		
		String[] registryKeys = registryList(location);
		for (int i = 0; i < registryKeys.length; i++)
		{
			String key = registryKeys[i];
			if (StringUtility.contains(key, newNamePrefix))
			{
				String postfixesCounter = key.substring(key.indexOf("-") + 1, key.length()); //$NON-NLS-1$
				existingPostfixesCounters.add(Integer.valueOf(postfixesCounter));
			}
		}

		int newPostfixCounter = 0;
		while(true)
		{
			if(!existingPostfixesCounters.contains(new Integer(newPostfixCounter)))
				return newNamePrefix + "-" + newPostfixCounter; //$NON-NLS-1$
			newPostfixCounter++;				
		}
	}

	protected String[] registryList(RegistryLocation registryLocation) 
	throws AccessException, RemoteException
	{
		return getRegistry(registryLocation).list();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.almendra.commons.cas.commands.NodeStartedInterpreter#checkCentralExists()
	 */
	protected boolean checkCentralExists(RegistryLocation registryLocation) throws AccessException, RemoteException
	{
		String[] keys = registryList(registryLocation);
		for (int i = 0; i < keys.length; i++)
		{
			if (keys[i].equals(CentralAnchor.RMI_NAME))
				return true;
		}
		return false;
	}

	/**
	 * @param objectSpace
	 */
	protected void registerWithCentral(CommandAccessor cmdAccessor) 
	throws NotBoundException, RemoteException, InvalidCommandException
	{
		NodeStartedCommand command = (NodeStartedCommand) cmdAccessor.getCommand();
		IAgentDispatcher agentDispatcher = getCentralDispatcher(command.getCentralRegistryLocation());
		NodeAnchor anchor = cmdAccessor.getNodeAnchorIfPrivilege();
		anchor.addForeignNode(agentDispatcher, command.getCentralNodeLocation());
		Node node = cmdAccessor.getNodeIfPrivilege();
		SystemApplication sysApp = node.getSystemApplication();
		NodeLocation nodeLocation = new NodeLocation(anchor.getName(), sysApp.getLocalRegistryLocation());
		RegisterNodeCommand registerNodeCommand = new RegisterNodeCommand(nodeLocation);
		sysApp.getCentralAgent().accept(registerNodeCommand);
	}

	/**
	 * @param objectSpace
	 */
	protected NodeAnchor installNodeAnchor(IObjectSpace os, NodeStartedCommand command)
	{
		Node node = command.getNode();
		NodeAnchor anchor = new NodeAnchor(node);
		os.put(NodeAnchor.LABEL, anchor);
		return anchor;
	}

	/**
	 * @param command
	 */
	protected void bind(NodeStartedCommand command, IObjectSpace os) 
	throws RemoteException
	{
		String nodeName = null;
		boolean success = false;
		while (!success)
		{
			RMIUtility.ensureRegistryExists(command.getLocalRegistryLocation());
			nodeName = getNewNodeName(command.getLocalRegistryLocation());
			success = primBind(nodeName, command);
		}
		// propagate new name to node and all other objects that need it
		NodeAnchor anchor = (NodeAnchor) os.get(NodeAnchor.LABEL); 
		anchor.setName(nodeName);
		anchor.setViewTitle(ViewTitlePrefix + nodeName);
	}

	protected boolean primBind(String nodeName, NodeStartedCommand command)
	throws RemoteException
	{
		boolean success = false;
		try
		{
			Registry registry = RMIUtility.getRegistry(command.getLocalRegistryLocation());
			registry.bind(Node.RMI_LABEL_PREFIX + nodeName, command.getNode().getAgentDispatcher());
			success = true;
		}
		catch (AlreadyBoundException e)
		{
			// Do not write exception to console. It only means that <nodeName> is already in use.
			// Returning false will make the calling method look for a new name not in use until sucessful.
		}
		return success;
	}

	protected IAgentDispatcher getCentralDispatcher(RegistryLocation registryLocation) 
	throws AccessException, RemoteException, NotBoundException
	{
		return (IAgentDispatcher) getRegistry(registryLocation).lookup(CentralAnchor.RMI_NAME);
	}

	protected Registry getRegistry(RegistryLocation registryLocation) throws RemoteException
	{
		return RMIUtility.getRegistry(registryLocation);
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.scheduling.IInterpreter#commandNames()
	 */
	public List<String> commandNames()
	{
		return new StringList(NodeStartedCommand.QualifiedName);
	}

}
