/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source code or
 * in binary form must reproduce the above copyright notice, and the disclaimer
 * that follows.
 * 
 * In addition, you are required that you include in the end-user documentation
 * provided with the redistribution and/or in the software itself an
 * acknowledgement equivalent to the following: "This product is based on the
 * Janet software developed by the JANET Project
 * (http://www.almendra.org/janet/)."
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE JDOM
 * AUTHORS OR THE PROJECT CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */

package org.almendra.janet.cas.node.interpreters;

import java.rmi.AccessException;
import java.rmi.AlreadyBoundException;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.rmi.registry.Registry;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.almendra.commons.util.collection.ArrayUtility;
import org.almendra.commons.util.collection.StringList;
import org.almendra.commons.util.inet.InetUtility;
import org.almendra.commons.util.inet.RMIUtility;
import org.almendra.commons.util.swing.BillboardPanelController;
import org.almendra.janet.cas.commands.NodeStartedCommand;
import org.almendra.janet.cas.events.ClusterEventRegistry;
import org.almendra.janet.cas.events.ClusterEventRegistryRemote;
import org.almendra.janet.cas.events.IClusterEventRegistry;
import org.almendra.janet.cas.node.ClusterEventRegistryDescriptor;
import org.almendra.janet.cas.node.ClusterSpaceDescriptor;
import org.almendra.janet.cas.node.ClusterSubObjectDescriptor;
import org.almendra.janet.cas.node.Node;
import org.almendra.janet.cas.node.NodeAnchor;
import org.almendra.janet.cas.node.gui.NodeView;
import org.almendra.janet.cas.spaces.ClusterSpace;
import org.almendra.janet.cas.spaces.ClusterSpaceRemote;
import org.almendra.janet.cas.spaces.ISerializableObjectSpace;


/**
 * @author Oliver
 *  
 */
public class NodeStartedTrait
{

	/**
	 * Creates a new <code>NodeStartedTrait</code> object.
	 */
	public NodeStartedTrait()
	{
		super();
	}

	public void bindClusterSubobjects(Node node, NodeStartedCommand command) 
	throws AccessException, RemoteException, NotBoundException
	{
		String hostName = InetUtility.getHostName();
		
		Map<String, ISerializableObjectSpace> clusterSpacesByName = bindClusterSpaces(command.getClusterSpaceDescriptors(), hostName);
		node.addClusterSpaces(clusterSpacesByName);
		
		Map<String, IClusterEventRegistry> clusterEventRegistriesByName = bindEventRegistries(command.getClusterEventRegistryDescriptors(), hostName);
		node.addClusterEventRegistries(clusterEventRegistriesByName);
	}
	
	protected Map<String, ISerializableObjectSpace> bindClusterSpaces(List<ClusterSpaceDescriptor> clusterSubobjectDescriptors, String localHostname) 
	throws AccessException, RemoteException, NotBoundException
	{
		Map<String, ISerializableObjectSpace> clusterSubobjects = new HashMap<String, ISerializableObjectSpace>();
		for(ClusterSubObjectDescriptor desc : clusterSubobjectDescriptors)
		{
			if(desc.getRegistryLocation().getHostName().toUpperCase().equals(localHostname.toUpperCase()))
			{
				new BindClusterSpace().bind(desc, clusterSubobjects);
			}
		}
		return clusterSubobjects;
	}
	
	protected Map<String, IClusterEventRegistry> bindEventRegistries(List<ClusterEventRegistryDescriptor> clusterSubObjectDescriptors, String localHostname) 
	throws AccessException, RemoteException, NotBoundException
	{
		Map<String, IClusterEventRegistry> clusterSubobjects = new HashMap<String, IClusterEventRegistry>(); 
		for(ClusterSubObjectDescriptor desc : clusterSubObjectDescriptors)
		{
			if(desc.getRegistryLocation().getHostName().toUpperCase().equals(localHostname.toUpperCase()))
			{
				new BindClusterEventRegistry().bind(desc, clusterSubobjects);
			}
		}
		return clusterSubobjects;
	}
	
	class BindClusterSobject
	{
		protected void lookupClusterSpace(Registry registry, Map<String, ISerializableObjectSpace> clusterSpaces, String clusterSpaceRMIName, String clusterSpaceName) 
		throws AccessException, RemoteException, NotBoundException
		{
			ISerializableObjectSpace clusterSpace = (ISerializableObjectSpace) registry.lookup(clusterSpaceRMIName);
			clusterSpaces.put(clusterSpaceName, clusterSpace);	
		}
		
		protected void lookupEventRegistry(Registry registry, Map<String, IClusterEventRegistry> clusterEventRegistries, String clusterEventRegistryRMIName, String clusterEventRegistryName) 
		throws AccessException, RemoteException, NotBoundException
		{
			IClusterEventRegistry clusterEventRegistry = (IClusterEventRegistry) registry.lookup(clusterEventRegistryRMIName);
			clusterEventRegistries.put(clusterEventRegistryName, clusterEventRegistry);	
		}
	}
	
	class BindClusterSpace extends BindClusterSobject
	{
		public Map<String, ISerializableObjectSpace> bind(ClusterSubObjectDescriptor desc, Map<String, ISerializableObjectSpace> clusterSubObjects) 
		throws AccessException, RemoteException, NotBoundException
		{
			Registry registry = RMIUtility.ensureRegistryExists(desc.getRegistryLocation());
			String clusterSpaceRMIName = ClusterSpace.RMI_PREFIX + desc.getName();				
			if(!ArrayUtility.contains(registry.list(), clusterSpaceRMIName))
			{	
				ClusterSpace clusterSpace = new ClusterSpace();
				try
				{
					registry.bind(clusterSpaceRMIName, new ClusterSpaceRemote(clusterSpace));
				}
				catch (AlreadyBoundException e)
				{
					lookupClusterSpace(registry, clusterSubObjects, clusterSpaceRMIName, desc.getName());
					return clusterSubObjects;
				}
				clusterSubObjects.put(desc.getName(), clusterSpace);
				return clusterSubObjects;
			}
			lookupClusterSpace(registry, clusterSubObjects, clusterSpaceRMIName, desc.getName());
			return clusterSubObjects;
		}
	}
	
	class BindClusterEventRegistry extends BindClusterSobject
	{
		public Map<String, IClusterEventRegistry> bind(ClusterSubObjectDescriptor desc, Map<String, IClusterEventRegistry> clusterSubObjects) 
		throws AccessException, RemoteException, NotBoundException
		{
			Registry registry = RMIUtility.ensureRegistryExists(desc.getRegistryLocation());
			String clusterSpaceRMIName = ClusterEventRegistry.RMI_PREFIX + desc.getName();				
			if(!ArrayUtility.contains(registry.list(), clusterSpaceRMIName))
			{	
				ClusterEventRegistry clusterEventRegistry = new ClusterEventRegistry();
				try
				{
					registry.bind(clusterSpaceRMIName, new ClusterEventRegistryRemote(clusterEventRegistry));
				}
				catch (AlreadyBoundException e)
				{
					lookupEventRegistry(registry, clusterSubObjects, clusterSpaceRMIName, desc.getName());
					return clusterSubObjects;
				}
				clusterSubObjects.put(desc.getName(), clusterEventRegistry);
				return clusterSubObjects;
			}
			lookupEventRegistry(registry, clusterSubObjects, clusterSpaceRMIName, desc.getName());
			return clusterSubObjects;
		}
	}
		
	
	/**
	 * 
	 */
	public void showGUI(NodeStartedCommand command)
	{
		if(command.isStartGUI())
			NodeView.open(command.getNode());
	}

	protected List<String> getMonitorMask()
	{
		List<String> mask = new StringList();
		
		mask.add(BillboardPanelController.BLANK_LINE);
		mask.add(NodeAnchor.NODE_NAME);
		
		mask.add(BillboardPanelController.BLANK_LINE);
		
		mask.add(NodeAnchor.LOCAL_COMMANDS_COUNT);
		mask.add(NodeAnchor.LOCAL_WAITING_COMMANDS_COUNT);
		mask.add(NodeAnchor.LOCAL_EXECUTING_COMMANDS_COUNT);
		
		mask.add(NodeAnchor.FOREIGN_COMMANDS_COUNT);
		mask.add(NodeAnchor.FOREIGN_WAITING_COMMANDS_COUNT);
		mask.add(NodeAnchor.EVICTED_WAITING_COMMANDS_COUNT);
		mask.add(NodeAnchor.FOREIGN_EXECUTING_COMMANDS_COUNT);
		mask.add(NodeAnchor.EVICTED_EXECUTING_COMMANDS_COUNT);
		
		return mask;
	}
	
	protected Map<String, String> getMonitorMaskDefaultValues(String nodeName)
	{
		Map<String, String> defaultValues = new HashMap<String, String>();
		
		defaultValues.put(NodeAnchor.NODE_NAME, nodeName);
		
		defaultValues.put(NodeAnchor.LOCAL_COMMANDS_COUNT, "0"); //$NON-NLS-1$
		defaultValues.put(NodeAnchor.LOCAL_WAITING_COMMANDS_COUNT, "0"); //$NON-NLS-1$
		defaultValues.put(NodeAnchor.LOCAL_EXECUTING_COMMANDS_COUNT, "0"); //$NON-NLS-1$
		
		defaultValues.put(NodeAnchor.FOREIGN_COMMANDS_COUNT, "0"); //$NON-NLS-1$
		defaultValues.put(NodeAnchor.FOREIGN_WAITING_COMMANDS_COUNT, "0"); //$NON-NLS-1$
		defaultValues.put(NodeAnchor.EVICTED_WAITING_COMMANDS_COUNT, "0"); //$NON-NLS-1$
		defaultValues.put(NodeAnchor.FOREIGN_EXECUTING_COMMANDS_COUNT, "1"); //$NON-NLS-1$
		defaultValues.put(NodeAnchor.EVICTED_EXECUTING_COMMANDS_COUNT, "0"); //$NON-NLS-1$
		
		return defaultValues;
	}

	
}
