/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source code or
 * in binary form must reproduce the above copyright notice, and the disclaimer
 * that follows.
 * 
 * In addition, you are required that you include in the end-user documentation
 * provided with the redistribution and/or in the software itself an
 * acknowledgement equivalent to the following: "This product is based on the
 * Janet software developed by the JANET Project
 * (http://www.almendra.org/janet/)."
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE JDOM
 * AUTHORS OR THE PROJECT CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */

package org.almendra.janet.cas.scheduling;

import java.io.Serializable;
import java.rmi.RemoteException;

import org.almendra.commons.util.exception.InvalidMessageException;
import org.almendra.janet.cas.agents.AgentProxy;
import org.almendra.janet.cas.agents.IAgent;
import org.almendra.janet.cas.node.AgentPath;
import org.almendra.janet.cas.node.IAgentDispatcher;
import org.almendra.janet.cas.node.IllegalAgentAccessException;
import org.almendra.janet.cas.node.Logger;
import org.almendra.janet.cas.node.NoSuchAgentException;
import org.almendra.janet.cas.node.interpreters.NullInterpreter;


/**
 * @author Oliver
 *  
 */
public abstract class CommandResponseEnvelope extends CommandEnvelope
{
	protected AgentPath replyToAgentPath = null;
	protected Serializable response = null;
	protected boolean executed = false;
	protected boolean back = false;
	protected String responseId = null;
	
	/**
	 * Creates a new <code>CommandResponseEnvelope</code> object.
	 */
	public CommandResponseEnvelope(AgentPath agentPath, ICommand command, Logger logger, String responseId, String nodeName)
	{
		super(agentPath, command, logger, nodeName);
		this.responseId = responseId;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.almendra.commons.janet.cas.logger.node.CommandEnvelope#setReply(org.almendra.commons.janet.cas.logger.node.CommandHistoryItem)
	 */
	public void setReplyItem(CommandHistoryItem item)
	{
		item.setAgentPath(destinationAgent);
		destinationAgent = replyToAgentPath;
	}

	/**
	 * @param replyObject
	 */
	protected void sendBack()
	{
		prepareSendBack();
		
		CommandHistoryItem item = getResponseHistoryItem(node.getName());
		item.setCommandName(getCommand().getQualifiedName());
		setReplyItem(item);
	
		IAgentDispatcher agentDispatcher = null;
	
		try
		{
			agentDispatcher = getAgentDispatcher(getQualifiedDestinationAgentName());
		}
		catch (NoSuchAgentException e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
			log(e);
			return;
		}
		catch (IllegalAgentAccessException e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
			log(e);
			return;
		}
	
		try
		{
			// TODO - should be sent to agent proxy not to the dispatcher.
			// Problem: existing agent proxy only accept commands and not command envelopes.
			// TODO - until cleaned up a log statement has to be added that a command was sent
			// same as in class AgentProxy
			AgentProxy.LogSentCommand(this, logger);
			aboutToBeSent();	
			setSendTime();
			setPredefinedInterpreter(new NullInterpreter());
			agentDispatcher.accept(this);
		}
		catch (RemoteException ex)
		{
			System.out.println(ex.getMessage());
			ex.printStackTrace();
			log(ex);
		}
	}

	/**
	 * 
	 */
	protected void prepareSendBack()
	{
		// redefine in subclass as appropriate
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.almendra.commons.janet.cas.logger.schedulers.CommandEnvelope#execute(org.almendra.commons.janet.cas.logger.schedulers.IInterpreter)
	 */
	public void execute(IInterpreter interpreter)
	{
		if (!executed)
		{
			super.execute(interpreter);
			executed = true;
			sendBack();
		}
		else
		{
			callHandler();
		}
	}

	/**
	 * <code>CommandEnvelope</code> has arrived at the initial sender agent. 
	 * Invoke callback handler to handle the response. 
	 */
	protected abstract void callHandler();

	/**
	 * @param replyToAgentPath
	 * The replyToAgentPath to set.
	 */
	public void setReplyToAgentPath(AgentPath replyToAgentPath)
	{
		this.replyToAgentPath = replyToAgentPath;
	}

	/**
	 * @return Returns the replyToAgentPath.
	 */
	public AgentPath getReplyToAgentPath()
	{
		return replyToAgentPath;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.logger.schedulers.CommandEnvelope#getSuffix()
	 */
	protected String getSuffix()
	{
		if(isBack())
			return getSpecificSuffix();
		return ""; //$NON-NLS-1$
	}
	
	protected abstract String getSpecificSuffix();
	
	public boolean isBack()
	{
		return back;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.logger.schedulers.CommandEnvelope#setReply(java.io.Serializable)
	 */
	public void setResponse(Serializable responseObject) throws InvalidMessageException
	{
		// Prevent non-sensical compiler warning caused by throws clause of super method.
		if(false)
			throw new InvalidMessageException();
		
		response = responseObject;
	}
	
	public boolean isSynchronousResponse()
	{
		return executed;
	}

	/**
	 * @param agent
	 */
	public void callHandler(IAgent agent)
	{
		setCurrentAgent(agent);
		callHandler();
	}
	
	/**
	 * @param back The back to set.
	 */
	public void setBack(boolean back)
	{
		this.back = back;
	}

}
