/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.almendra.org/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE JANET AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
package org.almendra.janet.cas.spaces;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;


/**
 * The <code>AbstractObjectSpace</code> class implements an objectSpace for all nodes
 * in a cluster where all agents in the cluster can insert, retrieve, or remove objects.
 * 
 * @author Oliver Plohmann
 * @version 1.00, 15.02.2004
 * @see ISerializableObjectSpace
 * @see ClusterSpaceRemote
 */

public class ClusterSpace extends AbstractObjectSpace implements ISerializableObjectSpace
{
	public final static String RMI_PREFIX = "Janet.clusterSpace."; //$NON-NLS-1$
	
	/**
	 * The map to store objects stored by the object space.  
	 */
	protected Map<String, Serializable> objects = new HashMap<String, Serializable>();
	
	/**
	 * The map to store <code>ISerializableAttacher</code> objects by the key of the object they are associated with.
	 */
	protected Map<String, ISerializableAttacher> attachers = new HashMap<String, ISerializableAttacher>();
	
	/**
	 * The map to store <code>ISerializableDetacher</code> objects by the key of the object they are associated with.
	 */
	protected Map<String, ISerializableDetacher> detachers = new HashMap<String, ISerializableDetacher>();
	
	/**
	 * Creates a new <code>ClusterSpace</code> object.
	 */
	public ClusterSpace()
	{
		super();
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.spaces.IObjectSpace#removeAttacher(java.lang.String)
	 */
	public ISerializableAttacher removeAttacher(String key)
	{
		accessProtection.lock();
		try
		{
		    return attachers.remove(key);
		}
		finally
		{
		    accessProtection.unlock();
		}
	}

	/*
	 * (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.spaces.ISerializableObjectSpace#removeDetacher(java.lang.String)
	 */
	public ISerializableDetacher removeDetacher(String key)
	{
		accessProtection.lock();
		try
		{
		    return detachers.remove(key);
		}
		finally
		{
		    accessProtection.unlock();
		}
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.spaces.ISerializableObjectSpace#attach(java.lang.String, java.io.Serializable, org.almendra.commons.janet.cas.spaces.ISerializableAttacher)
	 */
	public void attach(String key, Serializable attachment, ISerializableAttacher attacher)
	{
		accessProtection.lock();
		try
		{
		    Serializable object = objects.get(key);
			attacher.attach(attachment, object);  
		}
		finally
		{
		    accessProtection.unlock();
		}
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.spaces.ISerializableObjectSpace#put(java.lang.String, java.io.Serializable)
	 */
	public void put(String key, Serializable object)
	{
	    accessProtection.lock();
		try
		{
		    objects.put(key, object);
		}
		finally
		{
		    accessProtection.unlock();
		}
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.spaces.ISerializableObjectSpace#installDetacher(java.lang.String, org.almendra.commons.janet.cas.spaces.ISerializableDetacher)
	 */
	public void installDetacher(String key, ISerializableDetacher detacher)
	{
		accessProtection.lock();
		try
		{
		    detachers.put(key, detacher);
		}
		finally
		{
		    accessProtection.unlock();
		}
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.spaces.ISerializableObjectSpace#installAttacher(java.lang.String, org.almendra.commons.janet.cas.spaces.ISerializableAttacher)
	 */
	public void installAttacher(String key, ISerializableAttacher attacher)
	{
		accessProtection.lock();
		try
		{
		    attachers.put(key, attacher);
		}
		finally
		{
		    accessProtection.unlock();
		}
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.spaces.ISerializableObjectSpace#detach(java.lang.String, org.almendra.commons.janet.cas.spaces.ISerializableDetacher)
	 */
	public Serializable detach(String key, ISerializableDetacher detacher)
	{
		accessProtection.lock();
		try
		{
		    Serializable object = objects.get(key);
			return detacher.detach(object);
		}
		finally
		{
		    accessProtection.unlock();
		}
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.spaces.ISerializableObjectSpace#attach(java.lang.String, java.io.Serializable)
	 */
	public void attach(String key, Serializable attachment) throws NoInstalledAttacher
	{
		accessProtection.lock();
		try
		{
		    ISerializableAttacher attacher = attachers.get(key);
			if (attacher == null)
				throwNoInstalledAttacher(key);
			Serializable object = objects.get(key);
			attacher.attach(attachment, object);
		}
		finally
		{
		    accessProtection.unlock();
		}
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.spaces.ISerializableObjectSpace#remove(java.lang.String)
	 */
	public Serializable remove(String key)
	{
		accessProtection.lock();
		try
		{
		    return objects.remove(key);
		}
		finally
		{
		    accessProtection.unlock();
		}
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.spaces.ISerializableObjectSpace#detach(java.lang.String)
	 */
	public Serializable detach(String key) throws NoInstalledDetacher
	{
		accessProtection.lock();
		try
		{
		    Serializable object = objects.get(key);
			ISerializableDetacher detacher = detachers.get(key);
			if (detacher == null)
				throwNoInstalledDetacher(key);
			return detacher.detach(object);
		}
		finally
		{
		    accessProtection.unlock();
		}		
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.spaces.ISerializableObjectSpace#get(java.lang.String)
	 */
	public Serializable get(String key)
	{
		accessProtection.lock();
		try
		{
		    return objects.get(key);
		}
		finally
		{
		    accessProtection.unlock();
		}
	}
	
	protected boolean primContains(String key)
	{
		return objects.containsKey(key);
	}
}
